#include <e32base.h>
#include <e32cons.h>
#include <w32std.h>
#include <e32keys.h>

#include "ClickSvr.h"

const TUid  KFpPropCat = {0xEE21DEE6};

const TInt KAnimTimer_Short_Delay = 150000;
const TInt KAnimTimer_Midle_Delay = 250000;
const TInt KAnimTimer_Long_Delay  = 600000;

/////////////////////////////////////////////////////////////////

CAnimTimer::CAnimTimer( MTimerNotifier& aTimeOutNotify, TInt aTimerType)
: CTimer( EPriorityRealTime ),
iNotifier( aTimeOutNotify ),
iTimerType( aTimerType )
{
}

CAnimTimer* CAnimTimer::NewL(MTimerNotifier& aTimeOutNotify, TInt aTimerType)
{
   CAnimTimer *self = new ( ELeave ) CAnimTimer( aTimeOutNotify, aTimerType );
   CleanupStack::PushL( self );
   self->ConstructL();
   CleanupStack::Pop( self );
   return self;
}

CAnimTimer::~CAnimTimer()
{
   Cancel();
}

void CAnimTimer::ConstructL()
{
   CTimer::ConstructL();
   CActiveScheduler::Add( this );
}

void CAnimTimer::RunL()
{
   iNotifier.TimedOut(iTimerType);
}

/////////////////////////////////////////////////////////////////

CClickSvr::CClickSvr( TInt aNo ): iNo( aNo )
{
  iIsVoiceKeyPressed  = EFalse;
}


CClickSvr::~CClickSvr()
{
  delete iShortTimer;
  delete iMidleTimer;
  delete iLongTimer;

  iProperty.Close();
}


void CClickSvr::ConstructL( TAny* aArgs, TBool /* aHasFocus */ )
{
  User::LeaveIfError(iProperty.Attach(KFpPropCat, 1, EOwnerProcess));

  //   
  iFunctions->SetSync( MAnimGeneralFunctions::ESyncNone );
  iFunctions->SetInterval( 0 );

  iShortTimer = CAnimTimer::NewL( *this, TAnimTimer_Short);
  iMidleTimer = CAnimTimer::NewL( *this, TAnimTimer_Midle);
  iLongTimer  = CAnimTimer::NewL( *this, TAnimTimer_Long);

  iFunctions->GetRawEvents( ETrue );
}


void CClickSvr::FocusChanged( TBool aState )
{
}


void CClickSvr::Redraw()
{
}


void CClickSvr::Animate( TDateTime* /* aDateTime */ )
{
}


TInt CClickSvr::CommandReplyL( TInt aOpcode, TAny* /* aArgs */ )
{
  //  
  return aOpcode;
}


void CClickSvr::Command( TInt aOpcode, TAny* aArgs  )
{
  //      
  switch ( aOpcode )
  {
   case KVoiceUiChangeState:
    {
	iVoiceUi = *(TBool *)aArgs;
    } break;

   case KKeyPadLockStatus:
    {
	iKeyPadLock = *(TBool *)aArgs;
    } break;

   case KPlayerAliveStatus:
    {
	iPlayerAlive = *(TBool *)aArgs;
    } break;

   case KCaptureHwrKeys:
    {
	iCaptureHwrKeys = *(TBool *)aArgs;
    } break;

   case KMenuKeySimulateCancel:
    {
        iMenuKeySimulate = EFalse;
    } break;

   case KIsBelle_FP2:
    {
	iIsBelle_FP2 = *(TBool *)aArgs;
    } break;

   default:
    break;
  }
}


TBool CClickSvr::OfferRawEvent( const TRawEvent& aRawEvent )
{
   TBool aRet(EFalse);

   if(!iCaptureHwrKeys) return EFalse; // No capture!!!

   if(iMenuKeySimulate) return EFalse; // No capture!!!

   switch ( aRawEvent.Type() )
   {
    case TRawEvent::EKeyUp:
      switch(aRawEvent.ScanCode())
      {
       case EStdKeyDevice6: // VoiceCmdKey
        {
          if (iShortTimer->IsActive()) iShortTimer->Cancel();

          if (iIsVoiceKeyPressed)
          {	
            iIsVoiceKeyPressed = EFalse;

            iProperty.Set((iVoiceUi) ? EaDllResponse_None : EaDllResponse_VoiceCmdKey);
          }
        } break;

       case EStdKeyMenu: //Menu key
       case EStdKeyApplication0:
        {
          if(iKeyPadLock) // && iPlayerAlive)
          {
            if (iMidleTimer->IsActive()) iMidleTimer->Cancel();
            if (iLongTimer->IsActive()) iLongTimer->Cancel();

            if(iIsMenuKeyPressed)
            {
              iIsMenuKeyPressed = EFalse;
              iProperty.Set(EaDllResponse_MenuKey);
            }
            else if(iIsMenuKeyPressedLongTime)
            {
              iIsMenuKeyPressedLongTime = EFalse;
              iProperty.Set(EaDllResponse_MenuKeyLongTime);
            }
            else
            {
              iMenuKeySimulate = ETrue;

              iProperty.Set(aRawEvent.ScanCode());
            }

            aRet = ETrue; //Exclusive !!!
          }
        } break;

       case EStdKeyNo: //Red key
       case EStdKeyYes: //Green key
        {
          if(iKeyPadLock && iPlayerAlive)
          {
	    aRet = ETrue; //Exclusive !!!
	  }
        } break;

       case EStdKeyDevice7: //Camera key
        {
          if(iKeyPadLock && iPlayerAlive && !iIsBelle_FP2)
          {
	    aRet = ETrue; //Exclusive !!!
	  }
        } break;

       default:
         iProperty.Set( EaDllResponse_None );
      }
      break;

    case TRawEvent::EKeyDown:
      switch(aRawEvent.ScanCode())
      {
       case EStdKeyDevice6: // VoiceCmdKey
        {
          iIsVoiceKeyPressed = EFalse;

          iShortTimer->After( KAnimTimer_Short_Delay );
        } break;

       case EStdKeyNo: //Red key
        {
          if(iKeyPadLock && iPlayerAlive)
          {
	    iProperty.Set(EaDllResponse_RedKey);

	    aRet = ETrue; //Exclusive !!!
	  }
        } break;

       case EStdKeyYes: //Green key
        {
          if(iKeyPadLock && iPlayerAlive)
          {
	    iProperty.Set(EaDllResponse_GreenKey);

	    aRet = ETrue; //Exclusive !!!
	  }
        } break;

       case EStdKeyMenu: //Menu key
       case EStdKeyApplication0:
        {
          if(iKeyPadLock) // && iPlayerAlive)
          {
            iIsMenuKeyPressed = EFalse;
            iIsMenuKeyPressedLongTime = EFalse;

            if (iShortTimer->IsActive()) iShortTimer->Cancel();

            iMidleTimer->After( KAnimTimer_Midle_Delay );
            iLongTimer->After( KAnimTimer_Long_Delay );

	    aRet = ETrue; //Exclusive !!!
          }
        } break;

       case EStdKeyDevice7: //Camera key
        {
          if(iKeyPadLock && iPlayerAlive && !iIsBelle_FP2)
          {
	    iProperty.Set(EaDllResponse_CameraKey);

	    aRet = ETrue; //Exclusive !!!
	  }
        } break;

       default:
//         iProperty.Set( aRawEvent.ScanCode() );
         iProperty.Set( EaDllResponse_None );
      }
      break;

    default:
      break;
   }

   return aRet;
}


void CClickSvr::TimedOut(TInt aTimerType)
{
   switch ( aTimerType )
   {
    case TAnimTimer_Short:
    {
      iIsVoiceKeyPressed = ETrue;
    } break;

    case TAnimTimer_Midle:
    {
      iIsMenuKeyPressed = ETrue;
    } break;

    case TAnimTimer_Long:
    {
      iIsMenuKeyPressed = EFalse;
      iIsMenuKeyPressedLongTime = ETrue;
    } break;

   default:
    break;
   }


}


/////////////////////////////////////////////////////////////////

CAnimServerDll::CAnimServerDll() : CAnimDll()
{
}

CAnim* CAnimServerDll::CreateInstanceL( TInt aType  )
{
   //  ,  
   return ( new ( ELeave ) CClickSvr( aType ) );
}

EXPORT_C CAnimDll* CreateCAnimDllL()
{
   return ( new ( ELeave ) CAnimServerDll );
}
