#ifndef __ANIM_SERVER_H__
#define __ANIM_SERVER_H__

#include <w32adll.h>
#include <e32property.h>

const TInt KVoiceUiChangeState = 1001;
const TInt KKeyPadLockStatus = 1002;
const TInt KPlayerAliveStatus = 1003;
const TInt KMenuKeySimulateCancel = 1004;
const TInt KCaptureHwrKeys = 1005;
const TInt KIsBelle_FP2 = 1006;

enum TAniDllResponse
{
  EaDllResponse_None = 0,
  EaDllResponse_RedKey = 1,
  EaDllResponse_GreenKey = 2,
  EaDllResponse_MenuKey = 4,
  EaDllResponse_MenuKeyLongTime = 8,
  EaDllResponse_VoiceCmdKey = 0x10,
  EaDllResponse_CameraKey = 0x20,
  EaDllResponse_FocusKey = 0x40
};

///////////////////////////////////////////////////

enum TAnimTimerType
{
   TAnimTimer_Short,
   TAnimTimer_Midle,
   TAnimTimer_Long
};


class MTimerNotifier
{
   public:
      virtual void TimedOut(TInt aTimerType) = 0;
};


class CAnimTimer : public CTimer
{
   public:
      static CAnimTimer* NewL( MTimerNotifier& aTimeOutNotify, TInt aTimerType );
      virtual ~CAnimTimer();

   private: 
      CAnimTimer( MTimerNotifier& aTimeOutNotify, TInt aTimerType );
      void ConstructL();
      void RunL();

   private: 
      MTimerNotifier& iNotifier;
      TInt iTimerType;
};

//////////////////////////////////////////////////////////

class CClickSvr : public CWindowAnim, public MTimerNotifier
{
   public:
      CClickSvr( TInt aNo );
      virtual ~CClickSvr();

      void ConstructL( TAny* aArgs, TBool aHasFocus );
      TInt CommandReplyL( TInt aCommand, TAny* aArgs );
      void Command( TInt aCommand, TAny* aArgs );

      void Redraw();
      void Animate( TDateTime* aDateTime );
      void FocusChanged( TBool aState );          

      TBool OfferRawEvent( const TRawEvent &aRawEvent );

   private: 
      void TimedOut(TInt aTimerType);

   private: 
      TInt  iNo;

      TBool iVoiceUi;

      TBool iKeyPadLock;

      TBool iIsVoiceKeyPressed;

      TBool iIsMenuKeyPressed;
      TBool iIsMenuKeyPressedLongTime;
      TBool iMenuKeySimulate;

      TBool iPlayerAlive;

      TBool iCaptureHwrKeys;

      TBool iIsBelle_FP2;

      CAnimTimer* iShortTimer;
      CAnimTimer* iMidleTimer;
      CAnimTimer* iLongTimer;

      RProperty iProperty;
};


//       DLL
class CAnimServerDll : public CAnimDll
{
   public:
      CAnimServerDll();

   public:
      IMPORT_C CAnim* CreateInstanceL( TInt aType );
};


#endif // __ANIM_SERVER_H__
