#include <e32base.h>
#include <e32cons.h>

#include "DolbyHeadphone.h"

_LIT(DolbyCli,"dolbyheadphoneeffect.dll");
_LIT(EffectBaseCli,"effectbase.dll");


CMyDolbyHeadphone::CMyDolbyHeadphone(MMyDolbyHeadphoneObserver& aObs)
 :iObs(aObs)
{
   iOn = EFalse;
   iCreated = EFalse;
}


CMyDolbyHeadphone* CMyDolbyHeadphone::NewL(MMyDolbyHeadphoneObserver& aObs)
{
  CMyDolbyHeadphone* self = new (ELeave) CMyDolbyHeadphone(aObs);
  CleanupStack::PushL(self);
  self->ConstructL();
  CleanupStack::Pop(self);
  return self;
}


void CMyDolbyHeadphone::ConstructL()
{
  TLibraryFunction entry;

  User::LeaveIfError(EffectBaseLib.Load(EffectBaseCli));

  entry = EffectBaseLib.Lookup(1);
  iRegisterObs = reinterpret_cast<TRegisterObserverL>(entry);

  entry = EffectBaseLib.Lookup(2);
  iUnRegisterObs = reinterpret_cast<TUnRegisterObserver>(entry);


  User::LeaveIfError(DolbyHeadphoneLib.Load(DolbyCli));

  entry = DolbyHeadphoneLib.Lookup(15);
  Dolby15 = reinterpret_cast<TDolbyHeadphoneNewL15>(entry);

  entry = DolbyHeadphoneLib.Lookup(14); //!!!
  Dolby14 = reinterpret_cast<TDolbyHeadphoneNewL14>(entry);
}


void CMyDolbyHeadphone::CreateDolby1L(CMdaAudioPlayerUtility& aPlayer)
{
  if(iDolbyObj == NULL)
  {
    iDolbyObj = Dolby15(aPlayer);
    iRegisterObs(iDolbyObj, *this);
  }

  if(iDolbyObj) iCreated = ETrue;
}


void CMyDolbyHeadphone::CreateDolby2L(CMdaAudioOutputStream& aAudioOutput)
{
  if(iDolbyObj == NULL)
  {
    iDolbyObj = Dolby14(aAudioOutput);
    iRegisterObs(iDolbyObj, *this);
  }

  if(iDolbyObj) iCreated = ETrue;
}

__asm void Dolby_destructor(TAny* aR0)
{
        CODE16
         PUSH    {LR}
         LDR     R1, [R0]
         LDR     R1, [R1,#4]
         BLX     R1

         POP     {PC}
}

void CMyDolbyHeadphone::DestroyDolby()
{
  if (iDolbyObj)
  {
     iUnRegisterObs(iDolbyObj, *this);
     Dolby_destructor(iDolbyObj);

     iDolbyObj = NULL;

     iCreated = EFalse;
  }
}

CMyDolbyHeadphone::~CMyDolbyHeadphone()
{
  DestroyDolby();

  DolbyHeadphoneLib.Close();
  EffectBaseLib.Close();
}


__asm void DisableL(TAny* aR0) //CAudioEffect* aDolbyObj
{
        CODE16
         PUSH    {LR}
         LDR     R1, [R0]
         LDR     R1, [R1,#0x10]
         BLX     R1

         POP     {PC}
}

__asm void EnableL(TAny* aR0) //CAudioEffect* aDolbyObj
{
        CODE16
         PUSH    {LR}
         LDR     R1, [R0]
         LDR     R1, [R1,#0x14]
         BLX     R1

         POP     {PC}
}

__asm TBool HaveUpdateRights(TAny* aR0) //CAudioEffect* aDolbyObj
{
        CODE16
         PUSH    {LR}
         LDR     R1, [R0]
         LDR     R1, [R1,#0x1C]
         BLX     R1

         POP     {PC}
}


__asm TBool IsEnabled(TAny* aR0) //CAudioEffect* aDolbyObj
{
        CODE16
         PUSH    {LR}
         LDR     R1, [R0]
         LDR     R1, [R1,#0x20]
         BLX     R1

         POP     {PC}
}


void CMyDolbyHeadphone::DolbyOnL(TBool aOn)
{
  TBool aState(EFalse);

  iOn = aOn;

  if (iDolbyObj)
  {
    if(HaveUpdateRights(iDolbyObj))
    {
      if(iOn)
        EnableL(iDolbyObj);
      else
        DisableL(iDolbyObj);
    }

    aState = IsEnabled(iDolbyObj);
  }

  iObs.DolbyChange(aState);
}


TBool CMyDolbyHeadphone::IsCreated()
{
   return iCreated;
}


//MAudioEffectObserver
void CMyDolbyHeadphone::EffectChanged( const CAudioEffect* /*aObservedEffect*/, TUint8 aEvent )
{
   switch(aEvent)
   {
   case KEnabled:
   iObs.DolbyChange(ETrue);
   break;

   case KDisabled:
   iObs.DolbyChange(EFalse);
   break;

   case KGainedUpdateRights: // The effect has gained update rights
   TRAP_IGNORE(DolbyOnL(iOn));
   break;

   default:;	// KEnforced KNotEnforced KSpecificEffectBase KLostUpdateRights
   }
}
