#ifndef __FOLDERPLAYENGINE_h__
#define __FOLDERPLAYENGINE_h__

// INCLUDES
#include <Audio.h>
#include <MdaAudioOutputStream.h>

#include "EffectEngine.h"

#define KBoostValMax  4

// FORWARD DECLARATIONS
class CFolderView;

//APE--------------

#define APE_BLOCKS_PER_LOOP     512
#define APE_MAX_SEEKPOINTS      30000
#define APE_INPUT_CHUNKSIZE     (32*512)

#define APE_FILTER_HISTORY_SIZE 512

struct APE_buffers {
    TInt16 filterbuf64[(64*3 + APE_FILTER_HISTORY_SIZE) * 2]; /* 2432 or 4864 bytes */
    TInt16 filterbuf256[(256*3 + APE_FILTER_HISTORY_SIZE) * 2]; /* 5120 or 10240 bytes */
    TUint seektablebuf[APE_MAX_SEEKPOINTS];
    TInt  decoded0[APE_BLOCKS_PER_LOOP];
    TInt  decoded1[APE_BLOCKS_PER_LOOP];
    TUint8 inbuffer[APE_INPUT_CHUNKSIZE];
};

//WavePack-------------------------
#define WV_BUFFER_SIZE 0x8000 //32768

union Engine_buffers {
    APE_buffers  ape;
    TInt32 wv_buf[WV_BUFFER_SIZE];
};
//-------------------------------------------

class CPlayEngine : public CActive,
			public MMdaAudioOutputStreamCallback,
			public MMyDolbyHeadphoneObserver2
{
public:
	static CPlayEngine* NewL(CFolderView& aParent, RFs& aFs);

	virtual ~CPlayEngine();

	void PlayTheList();
	
	void Start();
	void Stop();
	void Pause();
	void Resume();

	void SetVolume(TInt aVol);

	void OutputBuffer();
	void ScheduleAudioReset();

	TInt read_filebuf(TUint8* pBuffer, int nBytesToRead);

protected:
	// MMdaAudioOutputStreamCallback
	virtual void MaoscOpenComplete(TInt aError);
	virtual void MaoscBufferCopied(TInt aError, const TDesC8& aBuffer);
	virtual void MaoscPlayComplete(TInt aError);

// engine
	bool ParseFile(const TDesC16& anFile);
	bool ParseFLAC();
	bool ParseMAC();
	bool ParseOgg();
	bool ParseWavPack();
	bool ParseMP3();
	void Flush();
	
	inline RBuf8& ActiveBuffer()
	{
		return iCurBuffer.Ptr() >= iEvenBuffer.Ptr() &&
						iCurBuffer.Ptr() < iEvenBuffer.Ptr() + iEvenBuffer.MaxLength() ? iEvenBuffer : iOddBuffer;
	}

	inline RBuf8& InactiveBuffer()
	{
		return iCurBuffer.Ptr() >= iEvenBuffer.Ptr() &&
						iCurBuffer.Ptr() < iEvenBuffer.Ptr() + iEvenBuffer.MaxLength() ? iOddBuffer : iEvenBuffer;
	}

private:
	// Constructors

	void ConstructL();

	CPlayEngine(CFolderView& aParent, RFs& aFs);

	// CActive	
	virtual void RunL();
	virtual void DoCancel();

	void DolbyChange2(TBool aState);

protected:
// GUI
	RBuf8				iEngineBuffers;

// audio device and its state
	CMdaAudioOutputStream*		iAudioOutput;
	TFourCC				iAudioDataType;
	TMdaAudioDataSettings		iAudioSettings;
	TBool				iSetAudioParam;		// time to reset the audio device
// double buffer	
	RBuf8				iEvenBuffer;
	RBuf8				iOddBuffer;
// thread synchronization	
	RThread				iMainThread, iAudioThread;
	RSemaphore			iFreeBuffer;
	RMutex				iExclusive;
	RCondVar			iStateChange;
	TBool				iAudioBusy, iBufferPending;

	RFs&				iFs;

	TBool				iLastFilePlayed;
public:
	CFolderView&			iParent;

	RFile				iAudioFile;
	TFourCC				iDataType;
	TInt				iSampleRate, iChannels;
	TPtr8				iCurBuffer;

	CAudioEffectEngine*	iAudioEffects;
	
	friend class CFolderView;
};

//////////////////////////////////////////////////////
inline TInt Freq2SR(TInt anFreq)
{
	switch(anFreq)
	{
		case 96000: return TMdaAudioDataSettings::ESampleRate96000Hz;
		case 64000: return TMdaAudioDataSettings::ESampleRate64000Hz;
		case 48000: return TMdaAudioDataSettings::ESampleRate48000Hz;
		case 44100: return TMdaAudioDataSettings::ESampleRate44100Hz;
		case 32000: return TMdaAudioDataSettings::ESampleRate32000Hz;
		case 24000: return TMdaAudioDataSettings::ESampleRate24000Hz;
		case 22050: return TMdaAudioDataSettings::ESampleRate22050Hz;
		case 16000: return TMdaAudioDataSettings::ESampleRate16000Hz;
		case 12000: return TMdaAudioDataSettings::ESampleRate12000Hz;
		case 11025: return TMdaAudioDataSettings::ESampleRate11025Hz;
		case 8000: return TMdaAudioDataSettings::ESampleRate8000Hz;
		default: return 0;
	}
}
#endif // __FOLDERPLAYENGINE_h__
// End of File
