// INCLUDE FILES
#include <flp_sdk_version.h>

#include <Audio.h>

#include <f32file.h>
#include <s32file.h>
#include <eikclbd.h>
#include <gulicon.h>
#include <aknconsts.h>
#include <akniconarray.h>
#include <bautils.h>
#include <utf.h>
#include <aknappui.h>
#include <akntitle.h>

#include <avkon.mbg>
#include <aknsconstants.h>

#include <AknsUtils.h> ///

#include <remconcoreapitarget.h>
#include <remconinterfaceselector.h>

#include "remconkeyresponse.h"

#include <mmf\common\mmfcontrollerpluginresolver.h>

#include "FolderPlayView.h"
#include "FolderPlayEngine.h"
#include "SimpleAudioPlayer.h"

#include "FolderPlayViewUi.h"

#include <FolderPlay.rsg>
#include <FolderPlay.mbg>

#include "FolderPlay.hrh"

#include <StringLoader.h>
#include <AknVolumePopup.h>
#include <sounddevice.h>

#include <centralrepository.h>

#include <mpxcommonplaybackview.mbg>

enum TMpxcommonplaybackviewToFold
{
EMbmFolderplayPlaying = 8,
EMbmFolderplayPaused = 9,
EMbmFolderplayRandom = 10,
EMbmFolderplayRepeat = 11,
EMbmFolderplayRepeat_one = 12
};

const TInt KFoldIconCount = 13;

_LIT(KPlayStateFile, "PlayState.ini");
_LIT(KSettingsFile, "FoldSettings.ini");
_LIT(KCUEext, ".cue");
_LIT(KFmt1, "%d\t");
_LIT(KFmt2, "\t%d");

const TChar KBackSlash = '\\';
const TChar KDot = '.';
const TChar KColon = ':';
const TChar KTabulate = '\t';
const TChar K_NewLine = '\n';

const TInt KFirstTimerExpiryInterval( 1 ); // Expire immediately
const TInt KTimerExpiryInterval( KAknStandardKeyboardRepeatRate );
const TInt KMPXMinVolume(0);
const TInt KMPXVolumeSteps(1);
const TInt KTenStepsVolume = 10;   
const TInt KTwentyStepsVolume = 20;

const TInt KUpdatePosDelay = 500000; // 0.5s
const TInt KUpdatePosDelay2 = 100000; // 0.1s

const TUid KCRUidMPXSettings = {0x101FFC03};
const TUint32 KMPXPlaybackVolume = 0x00000001; // Percentage of max volume

const TInt KTitleRefreshDelay = 40;
const TInt KVirtualRefreshDelay = 7; // 3,5sec

const TInt KPlayerAliveStatus = 1003; // For Inform Anim Dll - dsv

static const TUint encoding[] =
{
	KCharacterSetIdentifierUtf8,						// Test = 0,
	KCharacterSetIdentifierCodePage1252,		// UK English = 1,
	KCharacterSetIdentifierCodePage1252,		// French = 2,
	KCharacterSetIdentifierCodePage1252,		// German = 3,
	KCharacterSetIdentifierCodePage1252,		// Spanish = 4,
	KCharacterSetIdentifierCodePage1252,		// Italian = 5,
	KCharacterSetIdentifierCodePage1252,		// Swedish = 6,
	KCharacterSetIdentifierCodePage1252,		// Danish = 7,
	KCharacterSetIdentifierCodePage1252,		// Norwegian= 8,
	KCharacterSetIdentifierCodePage1252,		// Finnish = 9,
	KCharacterSetIdentifierCodePage1252,		// American English = 10,
	KCharacterSetIdentifierCodePage1252,		// Swiss French = 11,
	KCharacterSetIdentifierCodePage1252,		// Swiss German = 12,
	KCharacterSetIdentifierCodePage1252,		// Portuguese = 13,
	KCharacterSetIdentifierIso88593,		// Turkish = 14,
	KCharacterSetIdentifierCodePage1252,		// Icelandic = 15,
	KCharacterSetIdentifierIso88595,		// Russian = 16,
	KCharacterSetIdentifierIso88592,		// Hungarian = 17,
	KCharacterSetIdentifierCodePage1252,		// Dutch = 18,
	KCharacterSetIdentifierCodePage1252,		// Belgian Flemish = 19,
	KCharacterSetIdentifierCodePage1252,		// Australian English = 20,
	KCharacterSetIdentifierCodePage1252,		// Belgian French = 21,
	KCharacterSetIdentifierCodePage1252,		// Austrian German = 22,
	KCharacterSetIdentifierCodePage1252,		// New Zealand English = 23,
	KCharacterSetIdentifierCodePage1252,		// International French = 24,
	KCharacterSetIdentifierIso88592,		// Czech = 25,
	KCharacterSetIdentifierIso88592,		// Slovak = 26,
	KCharacterSetIdentifierIso88592,		// Polish = 27,
	KCharacterSetIdentifierIso88592,		// Slovenian = 28,
	KCharacterSetIdentifierBig5,			// Taiwanese Chinese = 29,
	KCharacterSetIdentifierBig5,			// Hong Kong Chinese = 30,
	KCharacterSetIdentifierGbk,			// Peoples Republic of China's Chinese = 31,
	KCharacterSetIdentifierShiftJis,		// Japanese = 32,
	KCharacterSetIdentifierUtf8,			// Thai = 33,
	KCharacterSetIdentifierUtf8,			// Afrikaans = 34,
	KCharacterSetIdentifierIso88592,		// Albanian = 35,
	KCharacterSetIdentifierUtf8,			// Amharic = 36,
	KCharacterSetIdentifierIso88596,		// Arabic = 37,
	KCharacterSetIdentifierUtf8,			// Armenian = 38,
	KCharacterSetIdentifierUtf8,			// Tagalog = 39,
	KCharacterSetIdentifierIso88595,		// Belarussian = 40,
	KCharacterSetIdentifierUtf8,			// Bengali = 41,
	KCharacterSetIdentifierIso88595,		// Bulgarian = 42,
	KCharacterSetIdentifierUtf8,			// Burmese = 43,
	KCharacterSetIdentifierCodePage1252,		// Catalan = 44,
	KCharacterSetIdentifierIso88592,		// Croatian = 45,
	KCharacterSetIdentifierCodePage1252,		// Canadian English = 46,
	KCharacterSetIdentifierCodePage1252,		// International English = 47,
	KCharacterSetIdentifierCodePage1252,		// South African English = 48,
	KCharacterSetIdentifierIso88594,		// Estonian = 49,
	KCharacterSetIdentifierIso88596,		// Farsi = 50,
	KCharacterSetIdentifierCodePage1252,		// Canadian French = 51,
	KCharacterSetIdentifierCodePage1252,		// Scots Gaelic = 52,
	KCharacterSetIdentifierUtf8,			// Georgian = 53,
	KCharacterSetIdentifierIso88597,		// Greek = 54,
	KCharacterSetIdentifierIso88597,		// Cyprus Greek = 55,
	KCharacterSetIdentifierUtf8,			// Gujarati = 56,
	KCharacterSetIdentifierIso88598,		// Hebrew = 57,
	KCharacterSetIdentifierUtf8,			// Hindi = 58,
	KCharacterSetIdentifierCodePage1252,		// Indonesian = 59,
	KCharacterSetIdentifierUtf8,			// Irish = 60,
	KCharacterSetIdentifierCodePage1252,		// Swiss Italian = 61,
	KCharacterSetIdentifierUtf8,			// Kannada = 62,
	KCharacterSetIdentifierIso88595			// Kazakh = 63,
};

	
CFolderView* CFolderView::NewL(CFolderPlayAppUi& aParent, RFs& aFs)
{
	CFolderView* self = new (ELeave) CFolderView(aParent, aFs);
	CleanupStack::PushL(static_cast<CCoeControl*>(self));
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


CDesC16ArrayFlat *CFolderView::GetSupportedHwrExtensionsL()
{
	CDesC16ArrayFlat *aExts(NULL);

	RMMFControllerImplInfoArray iControllers;

	// Defines the format support required
	CMMFFormatSelectionParameters* formSelParams=CMMFFormatSelectionParameters::NewLC();

	// Used to select controller plugins from the ECOM registry
	CMMFControllerPluginSelectionParameters* contPluginSelector=CMMFControllerPluginSelectionParameters::NewLC();

	// Call this method with 'blank' CMMFFormatSelectionParameters to select all controllers
	contPluginSelector->SetRequiredPlayFormatSupportL(*formSelParams);

	// List all controller plugins supported by the system
	contPluginSelector->ListImplementationsL(iControllers);
	CleanupStack::PopAndDestroy(2); // contPluginSelector, formSelParams

	if(iControllers.Count() > 0)
	{
	  aExts = new(ELeave) CDesC16ArrayFlat(4);

	  for(TInt aIdx = 0; aIdx < iControllers.Count(); ++aIdx)
	  {
	    const CMMFControllerImplementationInformation& ctrl = *iControllers[aIdx];

	    for(TInt aIdxF = 0; aIdxF < ctrl.PlayFormats().Count(); ++aIdxF)
	    {
		const CMMFFormatImplementationInformation& frmt = *ctrl.PlayFormats()[aIdxF]; 

		TBool aIsAudio(EFalse);

		const CDesC8Array& mimeTypes=frmt.SupportedMimeTypes();
		for(TInt k=0;k<mimeTypes.MdcaCount();k++)
		  if(mimeTypes.MdcaPoint(k).Find(_L8("audio/")) != KErrNotFound)
		  {
			aIsAudio = ETrue;
			break;
		  }

		if(!aIsAudio) continue;

		TBuf<12> exts;
		const CDesC8Array& file_ext=frmt.SupportedFileExtensions();
		for(TInt k=0;k<file_ext.MdcaCount();k++)
		{
			exts.Copy(file_ext.MdcaPoint(k).Mid(1));

			TInt aPos;
			if(aExts->Find(exts, aPos) == 0) continue;

			aExts->AppendL(exts);
		}
	    }
	  }
	}

	iControllers.ResetAndDestroy();

	return aExts;
}


CGulIcon* CreateColorIconL(
    MAknsSkinInstance* aSkin,
    const TAknsItemID& aIconSkinId,
    const TAknsItemID& aColorId,
    TInt aColorIndex,
    const TDesC& aBitmapFilename,
    TInt aFileBitmapId,
    TInt aFileMaskId,
    const TRgb aDefaultColor)
{
    CFbsBitmap* icon = NULL;
    CFbsBitmap* mask = NULL;

    AknsUtils::CreateColorIconLC(
        aSkin,           // aInstance
        aIconSkinId,     // aID
        aColorId,        // aColorID
        aColorIndex,     // aColorIndex
        icon,            // aBitmap
        mask,            // aMask
        aBitmapFilename, // aFilename
        aFileBitmapId,   // aFileBitmapId
        aFileMaskId,     // aFileBitmapId
        aDefaultColor ); // aDefaultColor

    CGulIcon* res = CGulIcon::NewL( icon, mask );
    res->SetBitmapsOwnedExternally( EFalse );
    CleanupStack::Pop(2); // mask, icon

    return res;
}


void CFolderView::ConstructL()
{
	CreateWindowL();

	iListBox.ConstructL(this, EAknListBoxSelectionList);
	iListBox.SetListBoxObserver(this);

#ifdef _FLP_SDK_VERSION_50_
	//enable marquee //!!!! 
	iListBox.ItemDrawer()->ColumnData()->EnableMarqueeL(ETrue);
#endif

	//enable scroll bars
	iListBox.CreateScrollBarFrameL(ETrue);
	iListBox.ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);


	TPtrC KMBMFile = _L("\\resource\\apps\\FolderPlay.mbm");
	
	TFileName MPXiconFile( KMPXCommonPlaybackViewIconFile );
	MAknsSkinInstance* skin = AknsUtils::SkinInstance();

	CArrayPtr<CGulIcon>* icons = new (ELeave) CAknIconArray(KFoldIconCount);
	CleanupStack::PushL(icons);
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayFile, EMbmFolderplayFile_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayFolder, EMbmFolderplayFolder_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayChip, EMbmFolderplayChip_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayRemovable, EMbmFolderplayRemovable_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayRemote, EMbmFolderplayRemote_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayHd, EMbmFolderplayHd_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayVirtual, EMbmFolderplayVirtual_mask));
	icons->AppendL(iEikonEnv->CreateIconL(KMBMFile, EMbmFolderplayTrack, EMbmFolderplayTrack_mask));

	CGulIcon* aIcon;

	// EMbmFolderplayPlaying = 8
        aIcon = CreateColorIconL(
                skin,
                KAknsIIDNone,
                KAknsIIDNone,//KAknsIIDQsnIconColors,
                -1,//EAknsCIQsnIconColorsCG26,
                MPXiconFile,
                EMbmMpxcommonplaybackviewQgn_prop_image_tb_play2,
                EMbmMpxcommonplaybackviewQgn_prop_image_tb_play2_mask,
		KRgbGreen
		);
        icons->AppendL(aIcon);

	// EMbmFolderplayPaused = 9
        aIcon = CreateColorIconL(
                skin,
                KAknsIIDNone,
                KAknsIIDNone,
                -1,
                MPXiconFile,
                EMbmMpxcommonplaybackviewQgn_prop_image_tb_pause,
                EMbmMpxcommonplaybackviewQgn_prop_image_tb_pause_mask,
		KRgbYellow
		);
        icons->AppendL(aIcon);

	// EMbmFolderplayRandom = 10
        aIcon = CreateColorIconL(
                skin,
                KAknsIIDNone,
                KAknsIIDNone,
                -1,
                MPXiconFile,
		EMbmMpxcommonplaybackviewQgn_indi_mup_random,
		EMbmMpxcommonplaybackviewQgn_indi_mup_random_mask,
//                EMbmMpxcommonplaybackviewQtg_toolbar_shuffle,
//                EMbmMpxcommonplaybackviewQtg_toolbar_shuffle_mask,
		KRgbCyan
		);
        icons->AppendL(aIcon);

	// EMbmFolderplayRepeat = 11
        aIcon = CreateColorIconL(
                skin,
                KAknsIIDNone,
                KAknsIIDNone,
                -1,
                MPXiconFile,
		EMbmMpxcommonplaybackviewQgn_indi_mup_repeat,
		EMbmMpxcommonplaybackviewQgn_indi_mup_repeat_mask,
//	        EMbmMpxcommonplaybackviewQtg_toolbar_repeat,
//	        EMbmMpxcommonplaybackviewQtg_toolbar_repeat_mask,
		KRgbCyan
		);
        icons->AppendL(aIcon);

	// EMbmFolderplayRepeat_one = 12
        aIcon = CreateColorIconL(
                skin,
                KAknsIIDNone,
                KAknsIIDNone,
                -1,
                MPXiconFile,
		EMbmMpxcommonplaybackviewQgn_indi_mup_repeat_one,
		EMbmMpxcommonplaybackviewQgn_indi_mup_repeat_one_mask,
//	        EMbmMpxcommonplaybackviewQtg_toolbar_repeat_one,
//	        EMbmMpxcommonplaybackviewQtg_toolbar_repeat_one_mask,
		KRgbCyan
		);
        icons->AppendL(aIcon);

	CleanupStack::Pop(icons); 
	iListBox.ItemDrawer()->ColumnData()->SetIconArray(icons);
	
	SetRect(iParent.ClientRect());

	TInt aProgressHeight = iListBox.ItemHeight();

	TRect rc = Rect();
	rc.iBr.iY -= aProgressHeight;
	iListBox.SetRect(rc);

	rc.iTl.iY = rc.iBr.iY; rc.iBr.iY += aProgressHeight;
	iMusicProgress = CMusicProgressBar::NewL( rc, this);

	iPlayEngine = CPlayEngine::NewL(*this, iFs);

	iInternalPlayer = CSimpleAudioPlayer::NewL(*this);

	iAutoResumeHandler = CAutoResumeHandler::NewL(*this);


	iAccMonitor=CAccMonitor::NewL();
	iAccMonitor->StartObservingL(this);

/////
    CMMFDevSound* devsound = CMMFDevSound::NewL();	
    iVolumeSteps = KTenStepsVolume; //Default 10 steps
    if ( devsound && devsound->MaxVolume() >= KTwentyStepsVolume ) 
        {
         iVolumeSteps = KTwentyStepsVolume; // set 20-steps volume
        }    
    delete devsound;

    // Popup volume control
    iVolPopup = CAknVolumePopup::NewL(NULL, ETrue);
    iVolPopup->SetObserver(this);
    iVolPopup->SetRange(KMPXMinVolume, iVolumeSteps);
    iVolPopup->SetStepSize(KMPXVolumeSteps);

    TInt aCurVol(0);
    TRAP_IGNORE(
      {
      CRepository* repository = CRepository::NewL( KCRUidMPXSettings );
      repository->Get( KMPXPlaybackVolume, aCurVol );
      delete repository;
      repository = NULL;
      } );

    // Ensure that initial value is set to popup. If not initialized, CAknVolumePopup::Value() returns 0 
    // but the control appears unmuted and allows volume to be decreased out of range (panics also with Avkon 6 in UDEB)
    iCurrentVol = (iVolumeSteps * aCurVol) / 100;

    HBufC* popupText = StringLoader::LoadLC( R_VOLUME_POPUP_TEXT );
    iVolPopup->SetTitleTextL( *popupText );
    CleanupStack::PopAndDestroy( popupText );

    // Timer for implementing repeat
    iTimer = CPeriodic::NewL( CActive::EPriorityStandard );
    iAccessoryTimer = CPeriodic::NewL( CActive::EPriorityStandard );
/////

	iRemSelector = CRemConInterfaceSelector::NewL();
	// owned by iRemSelector
	CRemConCoreApiTarget* iCoreTarget = CRemConCoreApiTarget::NewL(*iRemSelector, *this);

	if(iRemSelector && iCoreTarget)
	{
	  iResponseHandler = CRemConKeyResponse::NewL( *iCoreTarget );

	  iRemSelector->OpenTargetL();
	}


	iEqPresetNotifier = CEqPresetNotifyHandler::NewL(*this);
	iEqPresetNotifier->StartObservingL();

	iPositionUpdater = CPeriodic::NewL(CActive::EPriorityIdle);

//	CEikonEnv::Static()->SetSystem( ETrue ); // don't close me when the memory card is removed
	iDiskDetector = CDiskDetector::NewL(*this, iFs);   
	iDiskDetector->Start();

	iExtensions = iCoeEnv->ReadDesCArrayResourceL(R_SUPPORTED_EXTENSIONS);
	iInternalExts = GetSupportedHwrExtensionsL();

	iExtsForSearch = new (ELeave) CDesC16ArrayFlat(4);

//	TFileName aExt(_L("*."));

	for(TInt i = 0; i < iExtensions->Count(); i++)
	{
//	  aExt.SetLength(2);
//	  aExt.Append((*iExtensions)[i]);
//	  iExtsForSearch->AppendL(aExt);

	  iExtsForSearch->AppendL((*iExtensions)[i]);
	}

	for(TInt i = 0; i < iInternalExts->Count(); i++)
	{
//	  aExt.SetLength(2);
//	  aExt.Append((*iInternalExts)[i]);
//	  iExtsForSearch->AppendL(aExt);

	  iExtsForSearch->AppendL((*iInternalExts)[i]);
	}


	iCharConv = CCnvCharacterSetConverter::NewL();
	if((TUint)User::Language() >= sizeof(encoding) / sizeof(*encoding) ||
			iCharConv->PrepareToConvertToOrFromL(encoding[User::Language()], iFs) != CCnvCharacterSetConverter::EAvailable)
		iCharConv->PrepareToConvertToOrFromL(encoding[0], iFs);


	iLightStatus = CHWRMLight::ELightStatusUnknown;
	iLight = CHWRMLight::NewL(this);

	iTitlePane = static_cast<CAknTitlePane*>(iParent.StatusPane()->ControlL( TUid::Uid( EEikStatusPaneUidTitle )));

	LoadSettingsL();

	ActivateL();

	SetRootL();
}


CFolderView::CFolderView(CFolderPlayAppUi& aParent, RFs& aFs)
 :iParent(aParent),
  iFs(aFs),
  iPlayList(8)
{
	iCharConv = NULL;

	iPlayerType = EFFPtNone;
	
	iCurFileIdx = 0;
	iRemSelector = NULL;

	iAttribs = KEntryAttNormal | KEntryAttDir;
}


CFolderView::~CFolderView()
{
	if( iLight ) delete iLight;

	if(iCharConv) delete iCharConv;

	delete iExtsForSearch;

	delete iExtensions;
	delete iInternalExts;

	delete iDiskDetector;  

	if(iResponseHandler) delete iResponseHandler;

	if(iRemSelector) { delete iRemSelector; iRemSelector = NULL; }

	if( iAccessoryTimer )
        {
	        iAccessoryTimer->Cancel();
        	delete iAccessoryTimer;
        }

	if( iTimer )
        {
	        iTimer->Cancel();
        	delete iTimer;
        }

	delete iVolPopup;
	
	delete iPositionUpdater;

	delete iEqPresetNotifier;

	delete iAccMonitor;

	delete iAutoResumeHandler;

	if(iMusicProgress) delete iMusicProgress;

	delete iPlayEngine;

	delete iInternalPlayer;
}


void CFolderView::SizeChanged()
{
	TInt aProgressHeight = iListBox.ItemHeight();

	TRect rc = Rect(); rc.iBr.iY -= aProgressHeight;
	iListBox.SetRect(rc);

	rc.iTl.iY = rc.iBr.iY; rc.iBr.iY += aProgressHeight;

	if(iMusicProgress)
	{
	  iMusicProgress->SetRect(rc);
	  iMusicProgress->DrawDeferred();
	}
}


void CFolderView::FocusChanged(TDrawNow aDrawNow)
{
  if(iMusicProgress) iMusicProgress->SetForegroundL(IsFocused());

  if(IsFocused()) RefreshProgress(!iAlive);

  if(aDrawNow == EDrawNow) DrawNow(); else DrawDeferred();
}


void CFolderView::HandleListBoxEventL(CEikListBox* /*aListBox*/, TListBoxEvent aEventType)
{
	switch(aEventType)
	{
#ifdef _FLP_SDK_VERSION_BELLE_
		case EEventItemSingleClicked:
#endif
		case EEventEnterKeyPressed:
		{
			TFileName aName;
			TInt aEntryType = GetItemName(aName);

//		        if(iPath.Length() + aName.Length() > KMaxPath) break;

			aName.Insert(0, iPath);

			switch(aEntryType)
			{
			case EMbmFolderplayFile:
				if(iAlive && iCurFileIdx < iPlayList.Count() &&
				 iPlayList[iCurFileIdx].CompareF(aName) == 0)
				{
					ResumePause();
				}
				else
				{
					Stop();
					ResetPlayList(); // stopped!!!

					GetPlayListL(iPath);
					if(iPlayList.Find(aName, iCurFileIdx) == 0) Start(0);
				}
				break;
					
			case EMbmFolderplayFolder:
				aName.Append(KBackSlash);
				SetPathL(aName);
				break;
					
			case EMbmFolderplayVirtual:
				TRAPD(err, SetVirtualPathL(aName))
				if(err != 0)
				  SetPathL(iPath);
				else
				  iInCueSheetMode = ETrue;
				break;
				
			case EMbmFolderplayTrack:
				{
				TBool aRepeatly(iIsRepeatlyOne);

				Stop();
				ResetPlayList(); // may be not stopped!!!

				iIsRepeatlyOne = aRepeatly;

				GetPlayListL(iPath.Left(iPath.LocateReverse(KBackSlash) + 1));

				if(iPlayList.Find(iPath, iCurFileIdx) == 0)
				{
				  TInt aPos(0);
				  TInt aIdx = iListBox.CurrentItemIndex();

				  TRAPD(err, aPos = -GetTrackTimeL(iPath, aIdx))
				  if(err == 0 && aPos <= 0)
				  {
					Start(aPos);
					iRefreshTrackPosition = ETrue;
				  }
				}
				} break;

			default: break;
			}
		} break;
		
	default:;
	}
}


void CFolderView::Backward()
{
   if(iAlive && iCurFileIdx > 0)
   {
     Stop();
     --iCurFileIdx;
     Start(0);
   }
}


void CFolderView::Forward()
{
   if(iAlive && (iCurFileIdx + 1 < iPlayList.Count()))
   {
     Stop();
     ++iCurFileIdx;
     Start(0);
   }
}


TKeyResponse CFolderView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	if(aType == EEventKeyUp)
		switch(aKeyEvent.iScanCode)
		{
		case EStdKeyLeftArrow:
			if(iAlive) Backward();
			else if(iRepeat > 0) SeekEnd();
			return EKeyWasConsumed;
				
		case EStdKeyRightArrow:
			if(iAlive) Forward();
			else if(iRepeat > 0) SeekEnd();
			return EKeyWasConsumed;
		
		default: break;
		}	
	else if(aType == EEventKey && aKeyEvent.iRepeats > 0)
	{
		if(!iAlive || iTotalLength == 0) return EKeyWasConsumed;

		switch(aKeyEvent.iScanCode)
		{
		case EStdKeyLeftArrow:
			if(iRepeat == 0) SeekStart(EFFType);
			return EKeyWasConsumed;
					
		case EStdKeyRightArrow:
			if(iRepeat == 0) SeekStart(ERwType);
			return EKeyWasConsumed;
				
		default: break;
		}
	}
		
	return iListBox.OfferKeyEventL(aKeyEvent, aType);
}


void CFolderView::IncrementProgress(TInt aInc)
{
        iSeekPoint += aInc * iSeekRatio;

	if(iSeekPoint > iTotalLength)
		iSeekPoint = iTotalLength;
	else if(iSeekPoint < 0)
		iSeekPoint = 0; 

	if(iAlive && iTotalLength > 0 && iMusicProgress)
	{
	 iMusicProgress->UpdateTimeIndicatorsL(iSeekPoint, iTotalLength);
	 iMusicProgress->DrawDeferred();
	}
}


void CFolderView::RefreshPos()
{
	switch(iFFRwType)
	{
	//   
	case EFFType:
		IncrementProgress(1 + iRepeat / 2);
		break;

	//   
	case ERwType:
		IncrementProgress(-1 - iRepeat / 2);
		break;

	default: return;
	}

	++iRepeat;
}


TInt CFolderView::FastForwardRewindPosUpdateCallback(TAny* aThis)
{
	CFolderView* This = static_cast<CFolderView*>(aThis);

	This->RefreshPos();

	return ETrue;
}


void CFolderView::DoSeekStart()
{
	if(iAlive && iTotalLength > 0)
	{
	  if(iPositionUpdater->IsActive()) iPositionUpdater->Cancel();

	  iSeeking = ETrue;
	}
}

void CFolderView::SeekStart(TInt aFFRwType)
{
	iFFRwType = aFFRwType;

	iSeekPoint = iCurrentPos;

	DoSeekStart();

	TCallBack callBack(FastForwardRewindPosUpdateCallback, this);
	TTimeIntervalMicroSeconds32 aDelay(KUpdatePosDelay2);
	iPositionUpdater->Start(aDelay, aDelay, callBack);
}


void CFolderView::DoSeekEnd()
{
	if(iSeeking)
	{
	  TInt aPos = Progress();

	  if(iTotalLength > 0 && iCurFileIdx < iPlayList.Count())
	  {
		switch(iPlayerType)
		{
		case EFFPtExternal:
		{
		    TBool aIsPaused(iPause);

		    if(!aIsPaused) Pause();
	
		    iCurrentPos = aPos;

		    Resume();

		    if(!aIsPaused) StartPosRefresh();
		}
		break;

		case EFFPtInternal:
		{
		    Stop();

		    SetPlayingItemL(KNullDesC);

		    Start(aPos);
		}
		break;

		default:;
		}
	  }

	  iSeeking = EFalse; // must be there!
	}
}


//        
void CFolderView::SeekEnd()
{
	iPositionUpdater->Cancel();

	iFFRwType = EFFRwNone;
	iRepeat = 0;

	DoSeekEnd();
}


void CFolderView::ResumePause()
{
	if(iPause)
		Resume();
	else	Pause();
}


void CFolderView::PlayResumePauseL()
{
     if(iAlive)
	ResumePause();
     else
	Refresh_LoadL();
}


void CFolderView::MrccatoCommand(TRemConCoreApiOperationId aOperationId, TRemConCoreApiButtonAction aButtonAct)
{
	if (aOperationId == ERemConCoreApiFastForward || aOperationId == ERemConCoreApiRewind )
	{
		switch(aButtonAct)
		{
		case ERemConCoreApiButtonRelease:
	          SeekEnd();
		break;

		case ERemConCoreApiButtonPress:
		  SeekStart((aOperationId == ERemConCoreApiFastForward) ? EFFType : ERwType);
		break;

		default: break;
		}
	}
	else if((aOperationId == ERemConCoreApiVolumeUp || aOperationId == ERemConCoreApiVolumeDown))
	{
            iTimer->Cancel();

            iIncreaseVol = (aOperationId == ERemConCoreApiVolumeUp ? ETrue: EFalse);

            switch (aButtonAct)
            {
            case ERemConCoreApiButtonPress:
                { //Start Timer
                  iTimer->Start( KFirstTimerExpiryInterval,
                                 KTimerExpiryInterval,
                                 TCallBack( TimerCallback, this ) );
                } break;

            case ERemConCoreApiButtonClick:
                {
                  UpdateVolume();
                } break;

            case ERemConCoreApiButtonRelease:
            default: break;
            }
	}
	else if(aButtonAct == ERemConCoreApiButtonClick)
	{
		switch(aOperationId)
		{
		case ERemConCoreApiPlay: // Not Alive !!!
		case ERemConCoreApiPause:
		case ERemConCoreApiPausePlayFunction:
		  PlayResumePauseL();
		break;
			
		case ERemConCoreApiStop:
		  SyncStop(iAlive); // store !!!
		break;
		
		case ERemConCoreApiBackward:
		  Backward();
		break;
			
		case ERemConCoreApiForward:
		  Forward();
		break;
			
		default: break;
		}		
	}
    iResponseHandler->CompleteAnyKey( aOperationId );
}


// For resume play after pause
void CFolderView::MrccatoPlay(TRemConCoreApiPlaybackSpeed aSpeed, TRemConCoreApiButtonAction aButtonAct)
{
   if((aButtonAct == ERemConCoreApiButtonClick) && (aSpeed == ERemConCoreApiPlaybackSpeedX1))
   {
     PlayResumePauseL();
   }

   iResponseHandler->CompleteAnyKey( ERemConCoreApiPlay );
}


TInt CFolderView::CountComponentControls() const
{
	return 2;
}


CCoeControl* CFolderView::ComponentControl(TInt aIndex) const
{
   switch(aIndex)
   {
   case 0:
     return (CCoeControl*)&iListBox;

   case 1:
     return (CCoeControl*)iMusicProgress;

   default:
     return NULL;
   }
}


////////////////////////////////////////////////////////////////////////////////////////
// Helpers

TInt CFolderView::GetIconIdx(TBool aIsFolder)
{
	if (iPause)
	  return EMbmFolderplayPaused;
	else
	{
	  if (aIsFolder)
	  {
		if (iIsRandom)
		  return EMbmFolderplayRandom;
		else if (iIsRepeatly)
		  return EMbmFolderplayRepeat;
		if (iIsRepeatlyOne)
		  return EMbmFolderplayRepeat_one;
		else
		  return EMbmFolderplayPlaying;
	  }
	  else
	  {
		if (iIsRepeatlyOne)
		  return EMbmFolderplayRepeat_one;
		else
		  return EMbmFolderplayPlaying;
	  }
	}
}


void CFolderView::SetRootL(const TDesC& anFocus)
{
	TDriveList aDriveList; User::LeaveIfError(iFs.DriveList(aDriveList));

	CDesCArray& itemArray = *static_cast<CDesCArray*>(iListBox.Model()->ItemTextArray());
	itemArray.Reset();

	// mark the playing drive
	TChar aPlaying = iAlive && iCurFileIdx < iPlayList.Count() ? User::UpperCase(iPlayList[iCurFileIdx][0]) : 0;
		
	for(TInt aDrive = 0; aDrive < aDriveList.Length(); ++aDrive)
		if(aDriveList[aDrive])
		{
			TDriveInfo anInfo; iFs.Drive(anInfo, aDrive);

			TInt aIcon;

			switch(anInfo.iType)
			{
			case EMediaHardDisk:
				aIcon = EMbmFolderplayRemovable;
				if(anInfo.iDriveAtt & KDriveAttInternal)
				  aIcon = EMbmFolderplayHd; 
				break;

			case EMediaFlash:
			case EMediaNANDFlash:
				aIcon = EMbmFolderplayChip;
				break;

			case EMediaRemote:
				aIcon = EMbmFolderplayRemote;
				break;

			default : continue; // EMediaUnknown EMediaFloppy EMediaCdRom EMediaRam EMediaRom EMediaNotPresent
			}

			TBuf<20> aDriveName; iFs.GetDriveName(aDrive, aDriveName); aDriveName.TrimAll();
			TChar aDriveLetter; RFs::DriveToChar(aDrive, aDriveLetter); aDriveLetter = aDriveLetter.GetUpperCase();
			TBuf<32> aItemText; aItemText.Format(KFmt1, aIcon);
			if(aDriveName.Length() > 0)
				{ aItemText.Append(aDriveName); aItemText.AppendFormat(_L(" (%c:)"), (TUint)aDriveLetter); }
			else aItemText.AppendFormat(_L("Drive %c:"), (TUint)aDriveLetter);
			if(aDriveLetter == aPlaying) aItemText.AppendFormat(KFmt2, GetIconIdx(ETrue));

			itemArray.AppendL(aItemText);

			if(anFocus.Length() == 2 &&
				User::UpperCase(anFocus[0]) == aDriveLetter.GetUpperCase() && anFocus[1] == KColon)
					iListBox.SetCurrentItemIndex(itemArray.Count() - 1);
		}
	
	iPath.Zero(); iListBox.HandleItemAdditionL(); iListBox.DrawNow();
}


void CFolderView::SetPathL(const TDesC& anPath, const TDesC& anFocus)
{
	TInt aPathLen(anPath.Length());

	if(aPathLen == 0) { SetRootL(anFocus); return; }

	TInt aCurIdx(0);

	CDir* pEntryList(NULL);

	if(iFs.GetDir(anPath, iAttribs, EDirsFirst | ESortByName | EAscending, pEntryList) != KErrNone) return;

	CleanupStack::PushL(pEntryList);

// is this directory playing ?
	TPtrC16 aPlaying; bool aIsFolder = false;
	if(iAlive && iCurFileIdx < iPlayList.Count() && iPlayList[iCurFileIdx].Left(aPathLen).CompareF(anPath) == 0)
	{
		aPlaying.Set(iPlayList[iCurFileIdx].Mid(aPathLen));
		TInt aI = aPlaying.Locate(KBackSlash);
		if(aI != KErrNotFound) { aPlaying.Set(aPlaying.Left(aI)); aIsFolder = true; }
	}

	CDesCArray& itemArray = *static_cast<CDesCArray*>(iListBox.Model()->ItemTextArray());
	itemArray.Reset();

	TBool aSetCurrent(EFalse);
	TPath aSheet(anPath);
	TInt aCnt(pEntryList->Count());
	CDir& EntryList = *pEntryList;

	TFileName aItemText;

// iterate through the entries
	for(TInt aIndex = 0; aIndex < aCnt; ++aIndex)
	{
		const TEntry& aEntry(EntryList[aIndex]);

		aSheet.SetLength(aPathLen);

//	        if(aPathLen + aEntry.iName.Length() >= KMaxPath) continue;

		if(aEntry.IsDir())
		{
			aSheet.Append(aEntry.iName);
			aSheet.Append(KBackSlash);

			if(iFoldersScanning)
			{
			  TInt err(KErrNone); TBool ret(EFalse);
			  TRAP(err, ret = SoundFileExistL(aSheet));
			  if( err != KErrNone || !ret) continue;
			}

			aItemText.Format(KFmt1, EMbmFolderplayFolder);
		}
		else
		{
			TInt aExtPos = aEntry.iName.LocateReverse(KDot);
			TBool aCueMaybe(EFalse);

			if(aExtPos == KErrNotFound || !IsSupportedQ(aEntry.iName, aExtPos, aCueMaybe)) continue;

			if(aCueMaybe)
			{
			  aSheet.Append(aEntry.iName.Left(aExtPos));
			  aSheet.Append(KCUEext);
			  aItemText.Format(KFmt1, BaflUtils::FileExists(iFs, aSheet) ? EMbmFolderplayVirtual : EMbmFolderplayFile);
			}
			else
			  aItemText.Format(KFmt1, EMbmFolderplayFile);
		}

		aItemText.Append(aEntry.iName);

//		if(aPlaying.Length() > 0 && (bool)aEntry.IsDir() == aIsFolder && aEntry.iName.CompareF(aPlaying) == 0)
		if((bool)aEntry.IsDir() == aIsFolder && aEntry.iName.CompareF(aPlaying) == 0)
		{
			aItemText.AppendFormat(KFmt2, GetIconIdx(aEntry.IsDir()));
			aSetCurrent = ETrue;
		}

		itemArray.AppendL(aItemText);

		if(aEntry.iName.CompareF(anFocus) == 0 || aSetCurrent)
		{
			aCurIdx = itemArray.Count() - 1;
			aSetCurrent = EFalse;
		}
	}

	CleanupStack::PopAndDestroy(pEntryList);

	iListBox.SetCurrentItemIndex(aCurIdx);
	iListBox.HandleItemAdditionL();
	iListBox.DrawNow();

	iPath = anPath;
}


HBufC16* CFolderView::GetFileContent(const TDesC& anPath)
{
	RFile aFile; if(aFile.Open(iFs, anPath, EFileRead|EFileShareReadersOnly) != KErrNone) return NULL;
	TInt aSize; if(aFile.Size(aSize) != KErrNone) return NULL;
	
	TBuf8<3> aBOM; aFile.Read(aBOM, 3);
	if(aBOM.Length() >= 2 && aBOM[0] == 0xFF && aBOM[1] == 0xFE)
	{
		if(aSize < 4) { aFile.Close(); return NULL; }
		TInt aPos = 2; aFile.Seek(ESeekStart, aPos);
		HBufC16* aUnicode = HBufC16::NewMax((aSize - 2) >> 1); TPtr8 aPtr((TUint8*)aUnicode->Ptr(), aSize - 2);
		aFile.Read(aPtr, aSize - 2); aFile.Close();
		if(aPtr.Length() == aSize - 2) return aUnicode; else { delete aUnicode; return NULL; }
	}
	else if(aBOM.Length() >= 3 && aBOM[0] == 0xEF && aBOM[1] == 0xBB && aBOM[2] == 0xBF)
	{
		if(aSize < 4) { aFile.Close(); return NULL; }
		TInt aPos = 3; aFile.Seek(ESeekStart, aPos);
		RBuf8 aUTF8; aUTF8.Create(aSize - 3); aFile.Read(aUTF8, aSize - 3); aFile.Close();
		HBufC16* aUnicode = CnvUtfConverter::ConvertToUnicodeFromUtf8L(aUTF8);
		aUTF8.Close(); return aUnicode;
	}
	else
	{
		if(aSize == 0) { aFile.Close(); return NULL; }
		TInt aPos = 0; aFile.Seek(ESeekStart, aPos);
		RBuf8 aForeign; aForeign.Create(aSize); aFile.Read(aForeign, aSize); aFile.Close();
		
		HBufC16* aUnicode = HBufC16::NewMax(aForeign.Length());
		TPtr16 aPtr = aUnicode->Des(); TInt aState = CCnvCharacterSetConverter::KStateDefault;
		TInt aRem = iCharConv->ConvertToUnicode(aPtr, aForeign, aState); aForeign.Close();
		if(aRem == 0) return aUnicode; else { delete aUnicode; return NULL; }
	}
}


void CFolderView::SetVirtualPathL(const TDesC& anPath)
{
	TPath aSheetPath(anPath.Left(anPath.LocateReverse(KDot))); aSheetPath.Append(KCUEext);
	HBufC16* aSheetData = GetFileContent(aSheetPath); User::LeaveIfNull(aSheetData);
	CleanupStack::PushL(aSheetData);
	
	CDesCArray& itemArray = *static_cast<CDesCArray*>(iListBox.Model()->ItemTextArray());
	itemArray.Reset();
	
	// current progress in MS
	const TInt aCurTime = 
		iAlive && iCurFileIdx < iPlayList.Count() && iPlayList[iCurFileIdx].CompareF(anPath) == 0 ?
		  iCurrentPos : 0;
	
	TLex16 parser(*aSheetData); TUint aTrackNum = 0, aCurTrack = 1;//, aCurDist = (TUint)-1; 
	while(!parser.Eos())
	{
		TPtrC16 aCmd = parser.NextToken();
		if(aCmd.CompareF(_L("TRACK")) == 0)
		{
			parser.SkipSpace(); User::LeaveIfError(parser.Val(aTrackNum));
			if(aTrackNum != (TUint)itemArray.Count() + 1) User::Leave(KErrCorrupt);
		}
		else if(aTrackNum > 0 && aCmd.CompareF(_L("TITLE")) == 0)
		{
			parser.SkipSpace(); TInt aQuatation = _L("\"'«\x2018\x201C").Locate(parser.Get());
			if(aQuatation == KErrNotFound) User::Leave(KErrCorrupt);
			parser.Mark();
			
			aQuatation = parser.Remainder().Locate(_L("\"'»\x2019\x201D")[aQuatation]);
			if(aQuatation == KErrNotFound) User::Leave(KErrCorrupt);
			parser.Inc(aQuatation); if(parser.MarkedToken().Locate(K_NewLine) != KErrNotFound) User::Leave(KErrCorrupt);
			
			aSheetPath.Format(KFmt1, EMbmFolderplayTrack); aSheetPath.Append(parser.MarkedToken());
			itemArray.AppendL(aSheetPath);
		}
		else if(aTrackNum > 0 && aCmd.CompareF(_L("INDEX")) == 0)
		{
			parser.SkipSpace(); TUint aType; User::LeaveIfError(parser.Val(aType));
			if(aType == 1)
			{
				parser.SkipSpace();
				TUint aMinute; User::LeaveIfError(parser.Val(aMinute)); if(parser.Get() != KColon) User::Leave(KErrCorrupt);
				TUint aSecond; User::LeaveIfError(parser.Val(aSecond)); if(parser.Get() != KColon) User::Leave(KErrCorrupt);
				TUint aFrame; User::LeaveIfError(parser.Val(aFrame));

				TInt aTime = (TInt64)(aMinute * 60 + aSecond) * 1000 + aFrame / 75;
/*
				if((TUint)Abs(aCurTime - aTime) < aCurDist)
				{
				  aCurDist = Abs(aCurTime - aTime);
				  aCurTrack = aTrackNum;
				}
*/
				if(aTime <= aCurTime )
				{
				  aCurTrack = aTrackNum;
				}
			}
		}
		
		TInt aNewLine = parser.Remainder().Locate(K_NewLine); if(aNewLine == KErrNotFound) break;
		parser.Inc(aNewLine + 1);
	}
	
	CleanupStack::PopAndDestroy(aSheetData);

	iPath = anPath;

////
	TInt aIdx(aCurTrack - 1);

//	iListBox.SetCurrentItemIndex(aIdx);
	iListBox.HandleItemAdditionL();
	iListBox.DrawNow();

	if(iAlive && (iPlayList[iCurFileIdx].CompareF(iPath) == 0)) // && aCurTime > 0)
	{

		aSheetPath.Format(KFmt1, iPause ? EMbmFolderplayPaused : EMbmFolderplayPlaying);
		aSheetPath.Append(itemArray[aIdx].Mid(itemArray[aIdx].Locate(KTabulate) + 1));

		itemArray.Delete(aIdx);
		itemArray.InsertL(aIdx, aSheetPath);

		iListBox.SetCurrentItemIndexAndDraw(aIdx); // dsv
	}
////
}


TInt CFolderView::GetTrackTimeL(const TDesC& anPath, TInt anIndex)
{
	TPath aSheetPath(anPath.Left(anPath.LocateReverse(KDot))); aSheetPath.Append(KCUEext);
	HBufC16* aSheetData = GetFileContent(aSheetPath); User::LeaveIfNull(aSheetData);
	CleanupStack::PushL(aSheetData);
	
	TLex16 parser(*aSheetData); TInt aTime = -1; TUint aTrackNum = 0;
	while(!parser.Eos())
	{
		TPtrC16 aCmd = parser.NextToken();
		if(aCmd.CompareF(_L("TRACK")) == 0) { parser.SkipSpace(); User::LeaveIfError(parser.Val(aTrackNum)); }
		else if(aTrackNum == (TUint)anIndex + 1 && aCmd.CompareF(_L("INDEX")) == 0)
		{
			parser.SkipSpace(); TUint aType; User::LeaveIfError(parser.Val(aType));
			if(aType == 1)
			{
				parser.SkipSpace();
				TUint aMinute; User::LeaveIfError(parser.Val(aMinute)); if(parser.Get() != KColon) User::Leave(KErrCorrupt);
				TUint aSecond; User::LeaveIfError(parser.Val(aSecond)); if(parser.Get() != KColon) User::Leave(KErrCorrupt);
				TUint aFrame; User::LeaveIfError(parser.Val(aFrame));
				aTime = (aMinute * 60 + aSecond) * 1000 + aFrame * 1000 / 75;
				break;
			}
		}
		TInt aNewLine = parser.Remainder().Locate(K_NewLine); if(aNewLine == KErrNotFound) break;
		parser.Inc(aNewLine + 1);
	}
	CleanupStack::PopAndDestroy(aSheetData); return aTime;
}


TInt CFolderView::GetItemName(TDes& anName) const
{
	TInt anIndex = iListBox.CurrentItemIndex();

	CDesCArray& itemArray = *static_cast<CDesCArray*>(iListBox.Model()->ItemTextArray());
	if(anIndex >= itemArray.Count()) return -1;

	if(iPath.Length() > 0)
	{
		TLex16 aLex(itemArray[anIndex]);
		TInt aIcon; aLex.Val(aIcon); aLex.Inc(); aLex.Mark();
		TInt aTab = aLex.Remainder().Locate(KTabulate);
		if(aTab == KErrNotFound) anName = aLex.Remainder(); else { aLex.Inc(aTab); anName = aLex.MarkedToken(); }
		return aIcon;
	}
	else
	{
		anName.SetLength(2);
		anName[0] = itemArray[anIndex][itemArray[anIndex].Locate(KColon) - 1];
		anName[1] = KColon; return EMbmFolderplayFolder;
	}
}


void CFolderView::BackL()
{
	iInCueSheetMode = EFalse;
	iNeedRefreshForPlayNextInList = EFalse;

	if(iPath.Length() == 0) return;
	if(iPath[iPath.Length() - 1] == KBackSlash) iPath.SetLength(iPath.Length() - 1);
	
	TInt aIndex = iPath.LocateReverse(KBackSlash) + 1;
	SetPathL(iPath.Left(aIndex), iPath.Mid(aIndex));
}

/*
TBool CFolderView::SoundFileExistL(const TDesC16& anPath)
{
	TBool ret(EFalse);
	TPath aEntryPath(anPath);
	TInt aPathLen(anPath.Length());

	TInt err(KErrNone);

	RDir aDir;

	User::LeaveIfError(aDir.Open(iFs, anPath, iAttribs));
        CleanupClosePushL(aDir);

	TEntryArray *pArray = new(ELeave) TEntryArray();
	CleanupStack::PushL(pArray);

	TEntryArray& array=*pArray;

	do{
	  err = aDir.Read(array);

	  if (err==KErrNone || err==KErrEof)
	  {
	    TInt count=array.Count();

	    if (count==0) break;

	    TInt i=0;
	    while (i<count)
	    {
		const TEntry& aEntry(array[i++]);

		if(aEntry.IsDir())
		{
		   aEntryPath.SetLength(aPathLen);
		   aEntryPath.Append(aEntry.iName);
		   aEntryPath.Append(KBackSlash);

		   ret = SoundFileExistL(aEntryPath);
		}
		else
		{
		   ret = IsSupported(aEntry.iName);
		}
		if(ret) goto _found_ok;
	    }
	  }
	}while (err==KErrNone);

_found_ok:
	CleanupStack::PopAndDestroy(pArray);
	CleanupStack::PopAndDestroy(&aDir);

	if (!(err==KErrNone || err==KErrEof)) User::Leave(err);

	return ret;
}
*/

TBool CFolderView::SoundFileExistL(const TDesC16& anPath)
{
	TBool ret(EFalse);
	TPath aEntryPath(anPath);
	TInt aPathLen(anPath.Length());

	CDir* pEntryList(NULL);

	User::LeaveIfError(iFs.GetDir(anPath, iAttribs, EDirsLast, pEntryList));

	CleanupStack::PushL(pEntryList);

	TInt aCnt(pEntryList->Count());
	CDir& EntryList = *pEntryList;

	for(TInt aIdx = 0; aIdx < aCnt; ++aIdx)
	{
		const TEntry& aEntry(EntryList[aIdx]);

		if(aEntry.IsDir())
		{
		   aEntryPath.SetLength(aPathLen);
		   aEntryPath.Append(aEntry.iName);
		   aEntryPath.Append(KBackSlash);

		   ret = SoundFileExistL(aEntryPath);
		}
		else
		{
		  ret = IsSupported(aEntry.iName);
		}
		if(ret) break;
	}
	CleanupStack::PopAndDestroy(pEntryList);

	return ret;
}


void CFolderView::GetPlayListL(const TDesC16& anPath)
{
	TPath aEntryPath(anPath);
	TInt aPathLen(anPath.Length());

	CDir* aEntryList(NULL);

	if(iFs.GetDir(anPath, iAttribs, EDirsFirst | ESortByName | EAscending, aEntryList) != KErrNone) return;

	CleanupStack::PushL(aEntryList);

	TInt aCnt(aEntryList->Count());
	for(TInt aIndex = 0; aIndex < aCnt; ++aIndex)
	{
		const TEntry& aEntry((*aEntryList)[aIndex]);

		aEntryPath.SetLength(aPathLen);
		aEntryPath.Append(aEntry.iName);

		if(aEntry.IsDir())
		{
		  aEntryPath.Append(KBackSlash);
		  GetPlayListL(aEntryPath);
		}
		else if(IsSupported(aEntry.iName))
			iPlayList.AppendL(aEntryPath);
	}
	CleanupStack::PopAndDestroy(aEntryList); //delete aEntryList;
}


TBool CFolderView::IsSupported(const TDesC16& anName) const
{
	TInt aPos;
	TInt aExtPos = anName.LocateReverse(KDot);

	if(aExtPos == KErrNotFound) return EFalse;

	TFileName aExt(anName.Mid(aExtPos + 1));

	return TBool(iExtsForSearch->Find(aExt, aPos) == 0);
}


TBool CFolderView::IsSupportedQ(const TDesC16& anName, TInt aExtPos, TBool& aCueMaybe) const
{
	TInt aPos;
	TFileName aExt(anName.Mid(aExtPos + 1));

	aCueMaybe = TBool(iExtensions->Find(aExt, aPos)==0);

	return TBool(aCueMaybe || (iInternalExts->Find(aExt, aPos)==0));
}


void CFolderView::SetPlayingItemL(const TDesC& anPlayingItem)
{
	if(iInCueSheetMode) return;

	TPtrC16 aPlaying; bool aIsFolder = false;
	if(anPlayingItem.Left(iPath.Length()).CompareF(iPath) == 0)
	{
		aPlaying.Set(anPlayingItem.Mid(iPath.Length()));
		TInt aIndex = aPlaying.Locate(KBackSlash);
		if(aIndex != KErrNotFound) { aPlaying.Set(aPlaying.Left(aIndex)); aIsFolder = true; }
	}

	TBool aSelectItem(EFalse);
	TInt aIdx(0);

	TFileName aItemText;
	CDesCArray& itemArray = *static_cast<CDesCArray*>(iListBox.Model()->ItemTextArray());
	TInt itemCnt(itemArray.Count());

	for(TInt aIndex = 0; aIndex < itemCnt; ++aIndex)
	{
		TInt i = itemArray[aIndex].Locate(KTabulate);
		TLex aLex(itemArray[aIndex].Left(i)); TInt aIcon; aLex.Val(aIcon);
		
		TPtrC16 aFile(itemArray[aIndex].Mid(i + 1)); i = aFile.Locate(KTabulate);
		if(i != KErrNotFound)		// currently marked as playing
		{
			aFile.Set(aFile.Left(i));
			TInt aColon = aFile.Locate(KColon); if(aColon != KErrNotFound) aFile.Set(aFile.Mid(aColon - 1, 2));
			
			if((aIcon != EMbmFolderplayFile && aIcon != EMbmFolderplayVirtual) == aIsFolder && aFile.CompareF(aPlaying) == 0) continue;
			aItemText = itemArray[aIndex].Left(itemArray[aIndex].LocateReverse(KTabulate));
		}
		else										// currently marked as not playing
		{
			TInt aColon = aFile.Locate(KColon); if(aColon != KErrNotFound) aFile.Set(aFile.Mid(aColon - 1, 2));
			
			if((aIcon != EMbmFolderplayFile && aIcon != EMbmFolderplayVirtual) != aIsFolder || aFile.CompareF(aPlaying) != 0) continue;
			aItemText = itemArray[aIndex]; aItemText.AppendFormat(KFmt2, GetIconIdx(aIsFolder));

			aIdx = aIndex;
			aSelectItem = ETrue;
		}

		itemArray.Delete(aIndex);
		itemArray.InsertL(aIndex, aItemText);

//		if(aSelectItem)
//		{
//			iListBox.SetCurrentItemIndexAndDraw(aIndex); // dsv
//		}
//		else	iListBox.DrawItem(aIndex);
	}
	iListBox.DrawNow();
	if(aSelectItem)	iListBox.SetCurrentItemIndexAndDraw(aIdx); // dsv
}


void CFolderView::SetTotalLength(TInt aTotalLength)
{
	iTotalLength = aTotalLength;
	iSeekRatio = (TReal)iTotalLength / 1000;
}


TInt CFolderView::Progress() const
{
  if(iMusicProgress)
    return iMusicProgress->GetNewSongPosition();
  else
    return 0;
}


void CFolderView::StorePlayStateL()
{
	if(iPlayList.Count() == 0) return;
	
	TFileName aPlayStateFile;
	User::LeaveIfError(iFs.PrivatePath(aPlayStateFile));
	aPlayStateFile += KPlayStateFile;
	
	RFileWriteStream aPlayState;
	if(aPlayState.Open(iFs, aPlayStateFile, EFileWrite|EFileShareExclusive|EFileWriteBuffered) != KErrNone)
		User::LeaveIfError(aPlayState.Create(iFs, aPlayStateFile, EFileWrite|EFileShareExclusive|EFileWriteBuffered));

	if(iPlayerType == EFFPtExternal && iInternalPlayer) iCurrentPos = iInternalPlayer->RefreshPos();
  
	aPlayState.PushL();
  
	aPlayState.WriteUint8L(KFoldPlayStateVersion);	// version
	aPlayState.WriteInt32L(iIsRandom);
	aPlayState.WriteInt32L(iIsRepeatly);
	aPlayState.WriteInt32L(iIsRepeatlyOne);

	aPlayState.WriteInt32L(iCurrentPos);
	aPlayState.WriteInt32L(iCurFileIdx < iPlayList.Count() ? iCurFileIdx : 0);
	aPlayState.WriteInt32L(iPlayList.Count());
	for(TInt i = 0; i < iPlayList.Count(); ++i) aPlayState << iPlayList[i];
  
	aPlayState.CommitL(); aPlayState.Pop(); aPlayState.Release();
}


void CFolderView::LoadPlayStateL()
{
	TInt aPos;
	TFileName aPlayStateFile;
	User::LeaveIfError(iFs.PrivatePath(aPlayStateFile));
	aPlayStateFile += KPlayStateFile;
	
	RFileReadStream aPlayState;
	if(aPlayState.Open(iFs, aPlayStateFile, EFileRead|EFileShareReadersOnly|EFileReadBuffered) != KErrNone) return;

	aPlayState.PushL();
	if(aPlayState.ReadUint8L() == KFoldPlayStateVersion)
	{
		ResetPlayList(); //stopped!!!

		iIsRandom = (TBool)aPlayState.ReadInt32L();
		iIsRepeatly = (TBool)aPlayState.ReadInt32L();
		iIsRepeatlyOne = (TBool)aPlayState.ReadInt32L();

		// exclusive select
		if(iIsRandom)
		{
			iIsRepeatly = EFalse;
			iIsRepeatlyOne = EFalse;
		}
		else if(iIsRepeatly)
			iIsRepeatlyOne = EFalse;
	

		aPos = aPlayState.ReadInt32L();
		iCurFileIdx = aPlayState.ReadInt32L();
		TInt aCount = aPlayState.ReadInt32L();
		for(TInt i = 0; i < aCount; ++i)
		{
			TFileName aTrack; aPlayState >> aTrack;
			if(BaflUtils::FileExists(iFs, aTrack)) iPlayList.AppendL(aTrack);
			else if(iPlayList.Count() < iCurFileIdx) --iCurFileIdx;
			else if(iPlayList.Count() == iCurFileIdx) { iCurFileIdx = 0; aPos = 0; }
		}
		Start(aPos);
	}
	aPlayState.Pop(); aPlayState.Release();
}


void CFolderView::StoreSettingsL()
{
	TFileName aSettingsFile;
	User::LeaveIfError(iFs.PrivatePath(aSettingsFile));
	aSettingsFile += KSettingsFile;
	
	RFileWriteStream aSettings;
	if(aSettings.Open(iFs, aSettingsFile, EFileWrite|EFileShareExclusive|EFileWriteBuffered) != KErrNone)
		User::LeaveIfError(aSettings.Create(iFs, aSettingsFile, EFileWrite|EFileShareExclusive|EFileWriteBuffered));
  
	aSettings.PushL();
  
	aSettings.WriteUint8L(KFoldSettingVersion);	// version
	aSettings.WriteInt32L(iParent.iDolbyState);
	aSettings.WriteInt32L(iFoldersScanning);
	aSettings.WriteInt32L(iShowHiddingFolders);
	aSettings.WriteInt32L(iUseTitleBar);
	aSettings.WriteInt32L(iParent.iCaptureHwrKeys);
	aSettings.WriteInt32L(iCurrentVol);

	aSettings.WriteInt32L(iQuasiLameMP3);

	aSettings.WriteInt32L(iIsBoostVol);
  
	aSettings.CommitL();
	aSettings.Pop();
	aSettings.Release();
}


void CFolderView::LoadSettingsL()
{
	TFileName aSettingsFile;
	User::LeaveIfError(iFs.PrivatePath(aSettingsFile));
	aSettingsFile += KSettingsFile;
	
	RFileReadStream aSettings;
	if(aSettings.Open(iFs, aSettingsFile, EFileRead|EFileShareReadersOnly|EFileReadBuffered) != KErrNone) return;

	aSettings.PushL();
	if(aSettings.ReadUint8L() == KFoldSettingVersion)
	{
		iParent.iDolbyState = (TBool)(aSettings.ReadInt32L());
		iFoldersScanning = (TBool)(aSettings.ReadInt32L());
		iShowHiddingFolders = (TBool)(aSettings.ReadInt32L());
		iUseTitleBar = (TBool)(aSettings.ReadInt32L());

		iParent.iCaptureHwrKeys = (TBool)(aSettings.ReadInt32L());

		iCurrentVol = aSettings.ReadInt32L();
		iVolPopup->SetValue( iCurrentVol );

		iQuasiLameMP3 = (TBool)(aSettings.ReadInt32L());

		iIsBoostVol = (TBool)(aSettings.ReadInt32L());
		SetBoostVolVar();

		if(iShowHiddingFolders)
		  iAttribs = KEntryAttMatchMask;
		else
		  iAttribs = KEntryAttNormal | KEntryAttDir;
	}
	aSettings.Pop();
	aSettings.Release();
}


void CFolderView::Start(TInt aPos)
{
	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	if(iPlayList.Count() == 0) return;

	iPlayerType = EFFPtNone;

	TPtrC16 aCurClip(iPlayList[iCurFileIdx]);

	TInt aExtension = aCurClip.LocateReverse(KDot);

	if(aExtension != KErrNotFound)
	{
		iCurrentPos = aPos;

		TPtrC16 aType(aCurClip.Mid(aExtension + 1));

		TInt aP;
		if(iInternalExts->Find(aType, aP) == 0)
			iPlayerType = EFFPtExternal;
		else if(iExtensions->Find(aType, aP) == 0)
			iPlayerType = EFFPtInternal;


		// For Quasi-Gapless MP3 mode (original A.Demodov code)
		TBool aHideTime(EFalse);
		TBool aIsMP3(aType.CompareF(_L16("mp3")) == 0);

		if(iQuasiLameMP3 && aIsMP3)
		{
		  iPlayerType = EFFPtInternal;
		  aHideTime = ETrue;
		}

		if(iMusicProgress) iMusicProgress->HideTimeIndicators(aHideTime);

		if(iQuasiLameMP3_Changed && aIsMP3)
		{
		  iQuasiLameMP3_Changed = EFalse;
		  iCurrentPos = 0;
		}

		switch(iPlayerType)
		{
		case EFFPtExternal:
			iInternalPlayer->PlayL(aCurClip);
		break;

		case EFFPtInternal:
			iPlayEngine->Start();
		break;

		default:;
		}
	}
}


TInt CFolderView::PositionUpdateCallback(TAny* aThis)
{
	CFolderView* This = static_cast<CFolderView*>(aThis);

	This->RefreshProgress(EFalse);

	if(This->iBoostVolChange) This->ChangeVolumeBoost();

	return ETrue;
}


void CFolderView::StartPosRefresh()
{
	if(iPositionUpdater->IsActive()) iPositionUpdater->Cancel();

	TCallBack callBack(PositionUpdateCallback, this);
	TTimeIntervalMicroSeconds32 aDelay(KUpdatePosDelay);
	iPositionUpdater->Start(aDelay, aDelay, callBack);
}


void CFolderView::StartRefresh()
{
	if(iInCueSheetMode)
	  iRefreshTrackPosition = ETrue;
	else
	  SetPlayingItemL(iPlayList[iCurFileIdx]);

	RefreshTitle();

	StartPosRefresh();

	UpdatePlayPauseBtn();

	iAutoResumeHandler->HandlePlaybackStateChange(EFalse);
}


void CFolderView::Stop()
{
	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	switch(iPlayerType)
	{
	case EFFPtExternal:
		iInternalPlayer->Stop();
	break;

	case EFFPtInternal:
		iPlayEngine->Stop();
	break;

	default:;
	}

	iAutoResumeHandler->HandlePlaybackStateChange(EFalse);
}


void CFolderView::Pause()
{
	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	switch(iPlayerType)
	{
	case EFFPtExternal:
		iInternalPlayer->Pause();
	break;

	case EFFPtInternal:
		iPlayEngine->Pause();
	break;

	default:;
	}

	iAutoResumeHandler->HandlePlaybackStateChange(ETrue);
}

void CFolderView::Resume()
{
	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	switch(iPlayerType)
	{
	case EFFPtExternal:
		iInternalPlayer->Resume();
	break;

	case EFFPtInternal:
		iPlayEngine->Resume();
	break;

	default:;
	}

	iAutoResumeHandler->HandlePlaybackStateChange(EFalse);
}


void CFolderView::PauseResumeRefresh()
{
	if(iPause)
		iPositionUpdater->Cancel();
	else
		StartPosRefresh();

	if(iInCueSheetMode)
	{
	  TRAP_IGNORE(SetVirtualPathL(iPath));
	}
	else
	{
	  SetPlayingItemL(KNullDesC);
	  SetPlayingItemL(iPlayList[iCurFileIdx]);
	}

	UpdatePlayPauseBtn();
}


void CFolderView::ResetPlayList()
{
//	Stop();
	iPlayList.Reset();
	iCurFileIdx = 0;

	iIsRandom = EFalse;
	iIsRepeatly = EFalse;
	iIsRepeatlyOne = EFalse;
}


void CFolderView::SyncStop(TBool aStore)
{
	if(aStore) StorePlayStateL();

	Stop();

	StopRefresh();
}


void CFolderView::StopRefresh()
{
	iPositionUpdater->Cancel();

        if (iBoostVolChange)
	{
	  iBoostVolChange = EFalse;

          SetBoostVolVar();
	}

	if(iInCueSheetMode) BackL();

	SetPlayingItemL(KNullDesC);

	iTotalLength = iCurrentPos = 0;

	iIsRepeatlyOne = EFalse;

	RefreshProgress(ETrue);

	if(iTitlePane) iTitlePane->SetTextToDefaultL(); // Restore default title;

	UpdatePlayPauseBtn();

	iPlayerType = EFFPtNone;
}


void CFolderView::RefreshTitle()
{
	if(iTitlePane && iUseTitleBar)
	{
	  TPath aTitle(iPlayList[iCurFileIdx].Left(iPlayList[iCurFileIdx].LocateReverse(KDot)));
	  TPath aSheet(aTitle); aSheet.Append(KCUEext);

	  if( BaflUtils::FileExists(iFs, aSheet) )
		iTitlePane->SetTextL(aTitle, ETrue);
	  else
		iTitlePane->SetTextL(aTitle.Left(aTitle.LocateReverse(KBackSlash)), ETrue);
	}
}


void CFolderView::RefreshProgress(TBool aReset)
{
     static TInt aDelayCount;

     TBool aRefreshVirtual(EFalse);

     if(aReset)
     {
	iDelayScrollTitle = 0;

	if(iMusicProgress)
	{
	 iMusicProgress->UpdateTimeIndicatorsL(0, 0);
	 iMusicProgress->DrawDeferred();
	}
     }
     else if(IsFocused() && (iLightStatus == CHWRMLight::ELightOn))
     {
	if(iPlayerType == EFFPtExternal && iInternalPlayer)
		iCurrentPos = iInternalPlayer->RefreshPos();

	if(iAlive && iCurrentPos >= 0 && iCurrentPos < iTotalLength)
	{
	 if(iMusicProgress)
	 {
	   iMusicProgress->UpdateTimeIndicatorsL(iCurrentPos, iTotalLength);
	   iMusicProgress->DrawDeferred();
	 }

	 if(++iDelayScrollTitle > KTitleRefreshDelay)
	 {
	   iDelayScrollTitle = 0;

	   RefreshTitle();
	 }

	 if(iInCueSheetMode)
	 {
	   if(++aDelayCount > KVirtualRefreshDelay)
	   {
		TInt aPos(0);
		TInt aIdx = iListBox.CurrentItemIndex() + 1;

		aDelayCount = 0;

		CDesCArray& itemArray = *static_cast<CDesCArray*>(iListBox.Model()->ItemTextArray());

		if(aIdx < itemArray.Count())
		{
		  TInt err(KErrNone);
		  TRAP(err, aPos = GetTrackTimeL(iPath, aIdx))

		  if(err == KErrNone && aPos >= 0 && iCurrentPos > aPos)
			aRefreshVirtual = ETrue;
		}
	   }
	 }
	 else aDelayCount = 0;
	}

	if(iRefreshTrackPosition && iCurrentPos >= 0)
	{
	  iRefreshTrackPosition = EFalse;
	  aRefreshVirtual = ETrue;
	}

	if(aRefreshVirtual && (iPlayList[iCurFileIdx].CompareF(iPath) == 0))
	{
	  TRAP_IGNORE(SetVirtualPathL(iPath));
	}
     }
}


void CFolderView::UpdatePlayPauseBtn()
{
	if(iPlayPauseBtn)
	{
		if(iAlive && !iPause)
			iPlayPauseBtn->SetCurrentState(0, ETrue);
		else
			iPlayPauseBtn->SetCurrentState(1, ETrue);
	}
}


void CFolderView::SetVolume(TInt aVol)
{
	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	switch(iPlayerType)
	{
	case EFFPtExternal:
		iInternalPlayer->SetVolume(aVol);
	break;

	case EFFPtInternal:
		iPlayEngine->SetVolume(aVol);
	break;

	default:;
	}
}


// ---------------------------------------------------------------------------
// Callback for timer
// ---------------------------------------------------------------------------
TInt CFolderView::TimerCallback( TAny* aPtr )
{
    static_cast<CFolderView*>( aPtr )->UpdateVolume();
    return KErrNone;
}


void CFolderView::UpdateVolume()
{
    // Current volume level is not yet initialised
    if( iCurrentVol < 0 ) return;

    if ( iIncreaseVol )
    {
	if ( iMuted )
	{
		iMuted = EFalse;
		if ( iCurrentVol == 0 ) // prevent muting again when HandleControlEvent is called next time
                {
                   iCurrentVol = 1; // "1" is the first step of 20-step volume
                }
	}
	else
	{
		iCurrentVol = iCurrentVol < iVolumeSteps ? (iCurrentVol + 1) : iCurrentVol;
	}
	iVolPopup->SetValue( iCurrentVol );
    }
    else
    {
	if ( iCurrentVol == 1 )
	{
		iMuted = ETrue;
		iVolPopup->SetValue( 0 );
	}
	else if ( iMuted )	// Muted && volume > 1
	{
		iMuted = EFalse;
		iVolPopup->SetValue( iCurrentVol );
	}
	else
	{
		iCurrentVol = iCurrentVol - 1;
		iVolPopup->SetValue( iCurrentVol );
	}
    }

    if(iAvkonAppUi->IsForeground()) iVolPopup->ShowVolumePopupL();

    SetVolume(iCurrentVol);
}


void CFolderView::HandleControlEventL( CCoeControl* aControl, TCoeEvent aEventType )
{
	if( (aEventType == EEventStateChanged) && (aControl == iVolPopup) )
	{
		TInt vol = iVolPopup->Value();
		
		if ( vol == iCurrentVol && vol != 0 )
		{
			if ( !iMuted ) return;

		        iMuted = EFalse;
			SetVolume(iCurrentVol);
		}

		if ( vol == 0 )
		{
			if ( iMuted ) return;
			iMuted = ETrue;
			SetVolume(0);
		}
		else
		{
			if ( iMuted )
			{
				iMuted = EFalse;
				
				if ( iCurrentVol == 0 ) // setting volume to 0 would reapply mute, use slider value instead
				{
					iCurrentVol = vol;
				}
			}
			else
			{
				iCurrentVol = vol;
			}
			iVolPopup->SetValue( iCurrentVol );
			SetVolume(iCurrentVol);
		}
	}
}


TBool CFolderView::PlayNextInList()
{
   if(iAlive)
   {
     iTotalLength = iCurrentPos = 0;

     if(iPlayList[iCurFileIdx].CompareF(iPath) == 0)
       iNeedRefreshForPlayNextInList = iInCueSheetMode;

     ++iCurFileIdx;

     return ETrue;
   }
   else
     return EFalse;
}


void CFolderView::InformAniDll(TInt aCmd, TBool aPar)
{
   iParent.SendBoolCmdToAnimDll( aCmd, aPar);
}


void CFolderView::EqPresetChanged(TInt aPreset)
{
	if(!(iAlive && !iPause)) return;

	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	switch(iPlayerType)
	{
	case EFFPtExternal:
		TRAP_IGNORE(iInternalPlayer->iAudioEffects->SetEqualizerL(aPreset));
	break;

	case EFFPtInternal:
		TRAP_IGNORE(iPlayEngine->iAudioEffects->SetEqualizerL(aPreset));
	break;

	default:;
	}
}


TBool CFolderView::IsDolbySupported()
{
	return iParent.iDolbySupported;
}


TBool CFolderView::GetDolbyState()
{
	return iParent.iDolbyState;
}


void CFolderView::SetDolbyState(TBool aState)
{
	if(iPlayEngine == NULL || iInternalPlayer == NULL) return;

	if(!(iAlive && !iPause) || !IsFMTxPassive()) return;

	switch(iPlayerType)
	{
	case EFFPtExternal:
		TRAP_IGNORE(iInternalPlayer->iAudioEffects->SetDolbyHeadphoneL(aState));
	break;

	case EFFPtInternal:
		TRAP_IGNORE(iPlayEngine->iAudioEffects->SetDolbyHeadphoneL(aState));
	break;

	default:;
	}
}


void CFolderView::Refresh_LoadL()
{
	iInCueSheetMode	= iRefreshTrackPosition = EFalse;

	if(!iAlive) LoadPlayStateL();

	if(iCurFileIdx < iPlayList.Count())
		SetPathL(iPlayList[iCurFileIdx].Left(iPlayList[iCurFileIdx].LocateReverse(KBackSlash) + 1));
}


void CFolderView::ConnectedL(CAccMonitorInfo* aAccessoryInfo)
{
	TAccMonCapability cap(aAccessoryInfo->AccPhysicalConnection());

	if(cap==KAccMonBluetooth)
	{
	  iFirstTimeFlag = ETrue;
	}
}

void CFolderView::DisconnectedL(CAccMonitorInfo* aAccessoryInfo)
{
	TAccMonCapability cap(aAccessoryInfo->AccPhysicalConnection());
	
	if(iAlive && (cap==KAccMonHeadset ||
			 cap==KAccMonCarKit ||
			 cap==KAccMonLoopset ||
			 cap==KAccMonOffice ||
			 cap==KAccMonAVDevice ||
			 cap==KAccMonWired ||
			 cap==KAccMonBluetooth))
		SyncStop(iAlive); // store!!!

	if(cap==KAccMonBluetooth)
	{
	  iFirstTimeFlag = EFalse;
	}
}



TInt CFolderView::TimerCallbackAccesory( TAny* aPtr )
{
    TRAPD(err, static_cast<CFolderView*>( aPtr )->AccesoryChangedL());
    return err;
}

void CFolderView::AccesoryChangedL()
{
	if(iAlive) //1 
	{
		iAccAtomarMode = ETrue;

		SyncStop(EFalse); // no store!!!

		delete iPlayEngine;
		iPlayEngine = NULL;

		delete iInternalPlayer;
		iInternalPlayer = NULL;

	}
	else // 2  3 
	{
		if(iAccAtomarMode)
		{
			iPlayEngine = CPlayEngine::NewL(*this, iFs);
			iInternalPlayer = CSimpleAudioPlayer::NewL(*this);

			iAccAtomarMode = EFalse;
		}
		else
		{
			iAccessoryTimer->Cancel();

			Refresh_LoadL();
		}
	}
}

void CFolderView::AccesoryChangedAct(TInt aRestartTimeOut)
{
	if(iAlive && !iAccAtomarMode && iFirstTimeFlag)
	{
	  iAccessoryTimer->Cancel();
	  iAccessoryTimer->Start( KFirstTimerExpiryInterval, aRestartTimeOut, TCallBack( TimerCallbackAccesory, this ));

	  iFirstTimeFlag = EFalse;
	}
}


void CFolderView::UseTitleBarChangeL()
{
	iUseTitleBar = !iUseTitleBar;

	iDelayScrollTitle = 0;

	if(iAlive && iUseTitleBar) RefreshTitle();
	else if(iTitlePane) iTitlePane->SetTextToDefaultL(); // Restore default title
}


void CFolderView::FoldersScanningChangeL()
{
	iFoldersScanning = !iFoldersScanning;

	SetPathL(iPath);
}


void CFolderView::ShowHiddenFoldersChangeL()
{
	iShowHiddingFolders = !iShowHiddingFolders;

	if(iShowHiddingFolders)
	  iAttribs = KEntryAttMatchMask;
	else
	  iAttribs = KEntryAttNormal | KEntryAttDir;

	SetPathL(iPath);
}


void CFolderView::NotifyDiskChange()   
{
	iDiskDetector->Start();

	if(iAlive) SyncStop(iAlive); // store!!!

	SetRootL();
}   


void CFolderView::LightStatusChanged( TInt aTarget, CHWRMLight::TLightStatus aStatus )
{
    if ( aTarget == CHWRMLight::EPrimaryDisplay )
    {
	if ((aStatus == CHWRMLight::ELightOn || aStatus == CHWRMLight::ELightOff) && aStatus != iLightStatus)
    	{
		iLightStatus = aStatus;
	}
    }
}


void CFolderView::SetAlive(TBool aAlive)
{
    iAlive = aAlive;

    if(iMusicProgress) iMusicProgress->SetAlive(iAlive);

    InformAniDll(KPlayerAliveStatus, iAlive);
}


void CFolderView::ChangeMP3DecodeType()
{
    iQuasiLameMP3 = !iQuasiLameMP3;
    iQuasiLameMP3_Changed = !iQuasiLameMP3_Changed;
}


void CFolderView::ChangeBoostVol()
{
    iIsBoostVol = !iIsBoostVol;

    if(iAlive && !iPause && (iPlayerType == EFFPtInternal)) // played & (internal decoder)
    {
      iBoostVolChange = ETrue;

      if (iIsBoostVol)
      {
        iBoostVolCur = ETrue;
        iBoostVal = 1;
      }
      else
        if(--iBoostVal < 0) iBoostVal = 0;
    }
    else // stopped or paused
    {
       SetBoostVolVar();
    }
}

void CFolderView::SetBoostVolVar()
{
   iBoostVolCur = iIsBoostVol;

   if (iIsBoostVol)
     iBoostVal = KBoostValMax;
   else
     iBoostVal = 0;
}

void CFolderView::ChangeVolumeBoost()
{
  if (iIsBoostVol)
  {
    if (++iBoostVal > KBoostValMax) iBoostVal = KBoostValMax;
  }
  else
  {
    if (--iBoostVal < 0) iBoostVal = 0;

    if (iBoostVal == 0) iBoostVolCur = EFalse;
  }

  if (iBoostVal == 0 || iBoostVal == KBoostValMax) iBoostVolChange = EFalse;
}


TBool CFolderView::IsFMTxPassive()
{
    if(iParent.iFM_TX_Supported)
    {
      return !iParent.iFmTxActivity;
    }
    else return ETrue;
}


TInt CFolderView::RecalcPos(TUint32 aSampleRate)
{
    if(iCurrentPos < 0) iCurrentPos = -iCurrentPos;

    return (aSampleRate) ? (TUint64)aSampleRate * iCurrentPos / 1000 : iCurrentPos;
}

// End of File

/*
   _LIT(KFormat, "cPlay - %d %d");

   HBufC* buf = HBufC::NewLC(30);

   TPtr ptr = buf->Des();
   ptr.Format(KFormat, ...);

   CEikonEnv::Static()->InfoMsg(ptr);

   CleanupStack::PopAndDestroy(1);
*/
