#ifndef CMusicProgressBar_H
#define CMusicProgressBar_H

#include <coecntrl.h>

// CONSTANTS
_LIT(KMPXCommonPlaybackViewIconFile, "\\resource\\apps\\mpxcommonplaybackview.mif");

// FORWARD DECLARATIONS
class CEikLabel;
class MAknsControlContext;
class CFolderView;

/**  Duration display mode */
enum TMPXDuratDisplayMode
{
    EMPXDuratAuto = 0,
    EMPXDuratHMS
};

class CMusicProgressBar : public CCoeControl
{
public:
    static CMusicProgressBar* NewL( const TRect& aRect, const CCoeControl* aParent );

    virtual ~CMusicProgressBar();

    void UpdateTimeIndicatorsL(TInt aPlaybackPos, TInt aTotalLength);

    /**
     * Handles foreground events
     * @param aForeground ETrue if app is going to foreground, EFalse otherwise
     */
    void SetForegroundL( TBool aForeground );

    void SetAlive( TBool aAlive );

    void HideTimeIndicators(TBool aHide);

    /**
     * Gets song position (from touch progress bar)
     */
    TInt GetNewSongPosition();
    
protected:

    /**
     * Update progress bar graphics and redraw.
     * Refresh happens only when backlight is ON and UI is in foreground.
     */
    void RefreshProgressBar();

    /**
     * From CoeControl.
     * Handles a change to the control's resources.
     *
     * @param aType A message UID value.
     */
    void HandleResourceChange( TInt aType );

    /**
     * From CCoeControl.
     * Responds to changes to the size and position of the contents of this
     * control.
     */
    void SizeChanged();

    /**
     * From CCoeControl.
     * Gets the number of controls contained in a compound control.
     *
     * @return The number of component controls contained by this control.
     */
    TInt CountComponentControls() const;

    /**
     * From CCoeControl.
     * Gets an indexed component of a compound control.
     *
     * @param aIndex The index of the control.
     * @return The component control with an index of aIndex.
     */
    CCoeControl* ComponentControl( TInt aIndex ) const;

    /**
     * From CCoeControl.
     * Draws the control.
     *
     * @param aRect The rectangular region of the control to be drawn.
     */
    void Draw( const TRect& aRect ) const;

    /**
     * From CCoeControl.
     * Handle events from the touch screen
     */
    void HandlePointerEventL(const TPointerEvent& aPointerEvent);

private:

    void ConstructL( const TRect& aRect, const CCoeControl* aParent );

    void CreateIconsL();
    void FreeIcons();

    void UpdateLabelL( CEikLabel* aLabel, const TDesC& aText);

    void UpdateProgressBarGraphics();

    /**
     * Draw an icon
     *
     * @param aGc            Graphics context to draw to
     * @param aDirtyRect     Part of the screen that needs redrawing
     * @param aIndicatorRect Position for the icon
     * @param aIcon          Icon and mask
     * @param aInvertMask    Invert mask (see CBitmapContext::BitBltMasked)
     */
    void DrawIndicator(
        CWindowGc& aGc ,
        const TRect& aDirtyRect,
        const TRect& aIndicatorRect,
        const CGulIcon* aIcon,
        TBool aInvertMask = ETrue ) const;

    HBufC* DisplayableDurationL( TInt64 aDuration );

    void DoUpdateLayoutL();

    TInt CallSetPosition();
    static TInt CallSetPosition( TAny* aAny );

protected:   // data

    CEikLabel* iTextElapsedTime;   //dsv
    CEikLabel* iTextRemainingTime; //dsv
    CEikLabel* iTextTotalTime;     //dsv

    CGulIcon* iPlaybackSlider;          // owned
    CGulIcon* iSliderBackground;        // owned

    MAknsControlContext* iBackground;   // owned

    TRect iSliderPaneRect;
    TRect iSliderBackgroundRect;
    TRect iPlaybackSliderRect;

    TBool iAlive;

    TBool iHideTimeIndicators;

    // Progress bar position - playback
    TReal iPlaybackRatio;

    CPeriodic* iSeekTimer;

    TInt  iTrackTotalLength;
    TInt  iNewSongPosition;

    TMPXDuratDisplayMode iMode;

    TBool iTouchDown;        // Flag indicating touch progress bar is in use

    TBool iIsForeground;

    CFolderView*	iParent; // not owned
};

#endif  // CMusicProgressBar_H

// End of File
