#include "SimpleAudioPlayer.h"
#include "FolderPlayView.h"


// CONSTANTS
const TInt KOneSecInMS = 1000; // 1 second in milliseconds


CSimpleAudioPlayer::CSimpleAudioPlayer(CFolderView& aParent)
 :iParent(aParent)
{
}


CSimpleAudioPlayer::~CSimpleAudioPlayer()
{
        delete iAudioEffects;
	delete iPlayer;
}


CSimpleAudioPlayer* CSimpleAudioPlayer::NewL(CFolderView& aParent)
{
	CSimpleAudioPlayer* self = new (ELeave) CSimpleAudioPlayer(aParent);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}


void CSimpleAudioPlayer::ConstructL()
{
	iPlayer = CMdaAudioPlayerUtility::NewL(*this);
//	iPlayer->UseSharedHeap(); //31/01/13

	iAudioEffects = CAudioEffectEngine::NewL( iPlayer, iParent.IsDolbySupported(), *this);

	iStarting = EFalse;
}


void CSimpleAudioPlayer::PlayL(const TDesC& aFileName)
{
	TInt err(KErrNone);

	if(!iStarting)
		TRAP(err, iStarting = ETrue; iPlayer->OpenFileL(aFileName))

	if(err == KErrNotReady) 
	{
		iPlayer->Close();
		iStarting = EFalse;
	}
}


TInt CSimpleAudioPlayer::RefreshPos()
{
	TTimeIntervalMicroSeconds pos;
	iPlayer->GetPosition(pos);

	return (pos.Int64() / KOneSecInMS);
}


void CSimpleAudioPlayer::Pause()
{
	if(!iParent.iPause)
	{
		iParent.iPause = ETrue;

		iPlayer->Pause();

		if(!iParent.iSeeking) iParent.PauseResumeRefresh();

		iResumeRefresh = !iParent.iSeeking;
	}
}


void CSimpleAudioPlayer::Resume()
{
	if(iParent.iPause)
	{
		iParent.iPause = EFalse;

		if(iParent.iCurrentPos > 0 && iParent.iSeeking)
		{
		 TTimeIntervalMicroSeconds pos((TInt64)iParent.iCurrentPos * KOneSecInMS);
		 iPlayer->SetPosition(pos);
		}

		iPlayer->Play();

		if(iResumeRefresh) iParent.PauseResumeRefresh();
	}
}


void CSimpleAudioPlayer::Stop2()
{
	iAudioEffects->DestroyAudioEffect();

	iPlayer->Close();

	iParent.SetAlive(EFalse);
	iParent.iPause = EFalse;
}


void CSimpleAudioPlayer::Stop()
{
	if(iParent.iAlive)
	{
		iPlayer->Stop();
		Stop2();
	}
}


void CSimpleAudioPlayer::StopPost()
{
	Stop2();

	iParent.StopRefresh();
}


void CSimpleAudioPlayer::Start()
{
	iParent.iPause = EFalse;

	TInt err(KErrNone);
	TInt err2(KErrNone);

        TRAP( err,  iAudioEffects->CreateAudioEffectsL());

	if (iParent.IsDolbySupported() && iParent.IsFMTxPassive())
	{
	  TRAP(err2, iAudioEffects->SetDolbyHeadphoneL(iParent.GetDolbyState()));
	}

	if (iParent.iCurrentPos > 0)
	{
	  TTimeIntervalMicroSeconds pos((TInt64)iParent.iCurrentPos * KOneSecInMS);
	  iPlayer->SetPosition(pos);
	}

	iPlayer->Play();

        if( err != KErrNone || err2 != KErrNone)
        {
	  iAudioEffects->DestroyAudioEffect();

          TRAP(err, iAudioEffects->CreateAudioEffectsL());

 	  if (iParent.IsDolbySupported() && iParent.IsFMTxPassive())
	  {
	  TRAP(err2, iAudioEffects->SetDolbyHeadphoneL(iParent.GetDolbyState()));
	  }
        }
}


void CSimpleAudioPlayer::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& aDuration)
{
	if (KErrNone == aError)
	{
		SetVolume(iParent.iCurrentVol);

		iParent.SetTotalLength(aDuration.Int64() / KOneSecInMS);

		iParent.SetAlive(ETrue);

		Start();

		iParent.StartRefresh();
	}
	else
	{
		StopPost();
	}

	iStarting = EFalse;
}


void CSimpleAudioPlayer::MapcPlayComplete(TInt aError)
{
	if (KErrNone == aError)
	{
		if(iParent.iCurFileIdx + 1 < iParent.iPlayList.Count())
		{
			if(!iParent.PlayNextInList()) return;

_RepeatPlay:
			iAudioEffects->DestroyAudioEffect();

			iPlayer->Close();

			PlayL(iParent.iPlayList[iParent.iCurFileIdx]);
		}
		else if((iParent.iIsRepeatly || iParent.iIsRepeatlyOne) && iParent.iAlive)
		{
			iParent.iTotalLength = iParent.iCurrentPos = 0;

			iParent.iCurFileIdx = 0;

			goto _RepeatPlay;
		}
		else
		{
			StopPost();
		}
	}
	else
	{
	   switch (aError)
	   {
	   case KErrDied:
	   case KErrInUse:
	   case KErrAccessDenied:
		 iParent.iAutoResumeHandler->HandlePlaybackStateChange(EFalse);
		 iParent.iAutoResumeHandler->HandlePlaybackComplete(aError);
		 Pause();
		 break;

	   default:
		 StopPost();
	   }
	}
}


void CSimpleAudioPlayer::SetVolume(TInt aVol)
{
   if(aVol > 0)
	iPlayer->SetVolume(iPlayer->MaxVolume() * aVol / iParent.iVolumeSteps);
   else
	iPlayer->SetVolume(0);
}


void CSimpleAudioPlayer::DolbyChange2(TBool aState)
{
	iParent.iDolbyActive = aState;
}

// End of File
