#ifndef CAUTORESUMEHANDLER_H
#define CAUTORESUMEHANDLER_H

// INCLUDES
#include <e32base.h>
#include <mpxpskeyobserver.h>


// FORWARD DECLARATIONS
class CFolderView;
class CMPXPSKeyWatcher;

NONSHARABLE_CLASS(CAutoResumeHandler) :  public CBase,
                                         public MMPXPSKeyObserver
    {
public:
    static CAutoResumeHandler* NewL(CFolderView& aParent);

    virtual ~CAutoResumeHandler();
    
    void HandleOpenFileComplete();
    
    void HandlePlaybackStateChange(TBool aPaused);

    void HandlePlaybackComplete(TInt aError);

    void CancelResumeTimer();

private: // constructors

    CAutoResumeHandler(CFolderView& aParent);

    void ConstructL();

private: // From base classes

    /**
    * From MMPXPSKeyObserver
    * Handle PS event
    *
    * @param aUid The UID that identifies the property category
    * @param aKey The property sub-key
    */
    void HandlePSEvent(TUid aUid, TInt aKey);

private:  // New functions

    void DoHandleStateChangeL();

    TBool ShouldPause();

    void HandleResumeTimerCallback();

    static TInt ResumeTimerCallback(TAny* aPtr);

    void DoHandleVoiceCmdChange();

    void DoHandleAudioPolicyChangeL();

private: // data
    CFolderView&	iParent;

    TBool iPausedForCall;
    CMPXPSKeyWatcher* iStateObserver; // owned
    CMPXPSKeyWatcher* iTypeObserver;  // owned
    TTime iKErrDiedTime;

    // Timer for resume operation. Owned.
    CPeriodic* iResumeTimer;
    
    CMPXPSKeyWatcher* iVoiceCmdObserver;  // owned
    TBool iPausedForVoiceCmd;
    TBool iVoiceCmdResumeOngoing;

    // for tracking currently playing audio clients
    TInt iNumberOfActiveAudioClients;
    CMPXPSKeyWatcher* iAudioPolicyObserver; // owned

    TBool iPlayerOnPause;
    };

#endif      // CAUTORESUMEHANDLER_H   
            
// End of File
