/*
*
* Description:  Helper class for sending response back to RemCon Framework.
*
*/

#include "remconkeyresponse.h"


// ======== MEMBER FUNCTIONS ========

CRemConKeyResponse::CRemConKeyResponse( 
    CRemConCoreApiTarget& aRemConCoreApiTarget ) : 
    CActive( CActive::EPriorityStandard ),
    iRemConCoreApiTarget( aRemConCoreApiTarget )
{
    CActiveScheduler::Add( this );
}

CRemConKeyResponse* CRemConKeyResponse::NewL(
    CRemConCoreApiTarget& aRemConCoreApiTarget )
{
    CRemConKeyResponse* self = 
        new (ELeave) CRemConKeyResponse( aRemConCoreApiTarget );

    return self;
}

CRemConKeyResponse::~CRemConKeyResponse()
{
    Cancel();
    iResponseArray.Close();
}

// ---------------------------------------------------------------------------
// Send the any key response back to Remcon server
// ---------------------------------------------------------------------------
//
void CRemConKeyResponse::CompleteAnyKey( 
    TRemConCoreApiOperationId aOperationId )
{
    if ( !IsActive() )
    {
        switch ( aOperationId )
        {
           case ERemConCoreApiVolumeUp:
               {
               iRemConCoreApiTarget.VolumeUpResponse( iStatus, KErrNone );
               SetActive();
               break;
               }
           case ERemConCoreApiVolumeDown:
               {
               iRemConCoreApiTarget.VolumeDownResponse( iStatus, KErrNone );
               SetActive();
               break;
               }
           default:
               {
               TInt error = KErrNone;
               iRemConCoreApiTarget.SendResponse( iStatus, aOperationId, error );
               SetActive();
               break;
               }
        }
    }
    // already active. Append to array and complete later.
    else
    {
        iResponseArray.Append( aOperationId );
    }
}
    
void CRemConKeyResponse::DoCancel()
{
}

void CRemConKeyResponse::RunL()
{
    // if any existing -> Send response
    if ( iResponseArray.Count() )
    {
        CompleteAnyKey( iResponseArray[0] );
        // Remove already completed key
        iResponseArray.Remove( 0 );
        iResponseArray.Compress();
    }
}

// End of File
